#!/bin/ksh
# ---------------------------------------------------------------
# 3466 Network Storage Manager
#
# (C) COPYRIGHT International Business Machines Corp. 1998
# All Rights Reserved
# Licensed materials - Property of IBM
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM corp.
#
# 5765-D27 (C) Copyright IBM Corporation 1998 (Unpublished)
# ---------------------------------------------------------------
#
#  Component_name: prepare.ec 
#
#  Function: This script will prepare the 3466 boxes for the ec to upgrade to 
#    ADSM 3.7
#
#  Author  Tricia Jiang
#
#  other scripts need to run in conjuction:
#  /usr/lpp/adsmserv/bin/endadsm
#  /usr/lpp/adsmserv/bin/startadsm
#  captureIPenvironment            
#  restoreIPenvironment            
#  genucode
#  3494 microcode  ec048 
#  3590 microcode  ec048    
#  3590 Ultra microcode ec048 
#  7337 library   1.63/v96
#  7337 drive microcode 2560  
#  3570 and 3575:
#  MC3570B="422f 4234 ??"
#  MC3570C="52F1 5324 ??"
#  MC3575B="1.37 ??"
#  MC3575C="2.15 ??"
#  nsm=3.1.0.37
#  ssamc="17"  pjj check for 19 make sure upgrading
#
#  This script will log in:  /tmp/D19383A/prepare1.log 
#                            /tmp/D19383A/prepare2.log
#  The critical info will be stored in /tmp/D19383A/rec_log
#
#
#  Notes:  This script will write things to the rec_log
#  when the function completes successfully.  Info in the log is used
#  to skip sections when rerun is requested. It is also used to 
#  determine what options are included
#
#
#  To Rerun:
#     If you need to rerun from scratch with NO previous run info, phase2:
#       - delete /tmp/prepare directory and /tmp/record directory
#       - ensure migration threshold is set where you expect it (0) on
#             diskpools
#       - ensure reclamation threshold is set where you expect it on
#             tapepools
#     If you need to rerun from scratch with NO previous run info, phase3:
#       - delete same files as noted above
# ---------------------------------------------------------------------------
#  NOTE FOR FUTURE OS upgrades which use this script as a pattern
#       consider reworking the following areas!!!
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#  netbios/ipx - is there a file we can save that will rebuild these?
#  handle HSM being on the system. will exporint and importing handle hsm??
#  handle multiple servers
#  handle TIVOLI client being present. what needs to be saved off?
#  ssa adaptor code updates wont show up as loaded/updated until reboot done.
#  might have to do second ssadload if mc level way down level - check this out 
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
# pjj do we need to save off license information?
# pjj do we need to delete off old device drivers?
# pjj put in a verify installed section for critical files
# pjj make sure upgradedb gets done
# pjj make sure ADSM server(s) is/are up at end:

#  
#
# Change History
# ---------------------------------------------------------------------
# 7/29/99     Tricia Jiang      Initial Create
# 3/28/2000   Chris Knapp       Correct problem which prevents
#                               installation of web client updates
# ---------------------------------------------------------------------

export ECLEVEL="D19383A"
export ECLEVEL2="D19383A"
export prereqEC="F23306"   # could also be F23306a
export prereqEC2="F24317"  
export prereqEC3="F24196"
export prereqEC4="F23354"

export criticalFixes="tivoli.tsm.server.rte tivoli.tsm.devices.rte "

#export microDir=/storsmart/tricia/D19383A  #where the microcode will be read 
export tivolidir="/usr/tivoli/tsm/server/bin"
export microDir="/cd"    #where the microcode will be read 
export updateImageLoc="$microDir/usr/sys/inst.images"
export updateADSMLoc="$microDir/tsm"
export installADSMList="$updateADSMLoc/install.list"
export installList="$updateImageLoc/install.list"
export dirForFiles="/tmp/D19383A"
export dirForFilesInAdsmlog="/adsmlog/D19383A"
export dirForMc="/tmp/fwupdate"
export tsmSWAdir='tsmSWA.3.7.0.0'
export tsmclientdir="$microDir/tsmclient.3.7.0.0"
export targetCopyDir='/usr/nsm.3466/images'
export ADSMDOCdir="/usr/share/html/adsm"
export ADSMDOCdirold="/usr/share/html/adsm/winc"
export tsmDOCdir='tsmdoc.3.7.0.0/html/aix'
export tsmDocLink='/usr/lpp/internet/server_root/pub/adsmdocs'
export SSAdir='ssa'
export ssaTarFile='upgrade432.tar'

export StorageSmartLvl="3.1.0.6"
export OSLEVEL="4.3.2.0"
export ADSMLEVEL="3.7.1.0"
#export systemMCh50="wil99071.img"
#export systemMCh50="wil99253.img"
export systemMCh50="wil99308.img"
export systemLVLh50="L99308"
export systemMCh10="tr99181.img"
export systemLVLh10="R99181"
#export spMc="ag990315.img"
export spMc="ag990910.img"
export MC7337="q2_163.bin"
export MCdlt="DLT7000.HTI4.A0B00E24"
export MCdltlowercase="dlt7000.hti4.a0b00e24"

# the 3590, 3570, 3575 should be upgraded via their own ec.
export MC3494="LM520.05"
export MC3590a="4c6"
export MC3590b="88c"
export MC3590c="a70"

export MC3570B="4287"
export MC3570C="5424"

export MC3575B="1.37"
export MC3575C="2.15"


rec_log=$dirForFiles/D19383A_rec_log      # record necessary statistics
todo=$dirForFiles/to_do.log
consolePhase1=$dirForFiles/D19383A.console1
consolePhase2=$dirForFiles/D19383A.console2

model_file="sysInfo/3466.model"           # whatmodel file

reRunScript=''
phase=''
admInf="-id=admin -pass=admin -server=ss-m"
export man_pool=0

#---------------------------------------------------------------------------
#     get a valid ADSM admin id and password
#---------------------------------------------------------------------------
function admin_id {
   if [ "$reRunScript" = "yes" -a "$VERIFY" != "YES" ]
   then
      str=''
      str=$(cat $rec_log | grep ^"-id=" )
      if [ "$str" != '' ]
      then
         return 0
      fi
   fi
   echo "INFO: entering admin_id function" | tee -a $log

   srv=$1
   valid_id=0
   while [ valid_id -lt 1 -o valid_id -gt 1 ]
   do
      clear
      echo
      echo "*************************************************************************"
      echo "For ADSM server $srv:"
      echo "Please enter ADSM admin id that has system privileges:" | tee -a $log
      read adminid
      echo
      echo "*************************************************************************"
      echo "Please enter a valid password for $adminid:" | tee -a $log
      read adminpw
      echo
      echo "*************************************************************************"
      echo "Is $adminid with password $adminpw correct?" | tee -a $log
      echo "   1 ) YES: $adminid with pw $adminpw is correct."
      echo "   2 ) NO: please allow CE to reenter the id and password."
      echo
      read valid_id
      case $valid_id in
      1) echo;;
      2) echo;;
      *) let valid_id=0;;
      esac
   done
   ss=$(grep "$ssm " $clientdir/dsm.sys)
   if [ "$ss" = '' ]
   then
     # ----------------------------------------------------------------------
     #  Add in if not there
     # ----------------------------------------------------------------------
     echo "SErvername  $ssm " >> $clientdir/dsm.sys
     echo "   COMMmethod         sharedmem" >> $clientdir/dsm.sys
     echo "   TXNBYTE            2000" >> $clientdir/dsm.sys
     echo "   compression        off" >> $clientdir/dsm.sys
     echo "   shmport            $shmport" >> $clientdir/dsm.sys
     serve="$ssm"
   else
     serve="$ssm"
   fi

   export admInf="-id=$adminid -pass=$adminpw -se=$ssm"
   thr=$(cat $rec_log | grep ADMINF$srv | awk '{ print $2 }' )
   if [ "$thr" = '' ]
   then
      echo "ADMINF$srv $admInf " >> $rec_log
   fi
   echo "INFO: leaving admin_id function" | tee -a $log
}


# --------------------------------------------------------------------
# admin_info:  Obtain admin id and password and verify we can use these
#              to talk to the server.  This is used by phase2 only.
# --------------------------------------------------------------------
function admin_info {

   echo "INFO:  Entering admin_info" | tee -a $log

   valid_iv_st=''

   for server in $server_list
   do
      thr=$(cat $rec_log | grep "ADMINF$server ")
      if [ "$thr" = '' ]
      then
        adsm_able
      else
        echo "INFO: already have admin id for this server."
      fi
   done

   thr=$(echo $server_list | grep primary)
   if [ "$thr" = '' ]
   then
      server_list="primary "
   fi
   export server_list=$server_list
   thr=$(cat $rec_log | grep ^server_list)
   if [ "$thr" = '' ]
   then
      echo "server_list $server_list" >> $rec_log
   fi
   echo "INFO: leaving admin_info" | tee -a $log
}

# -------------------------------------------------------------------------
#    adsm_able make sure adsm server is able to come up - ie 2 db logs are
#         not down
# -------------------------------------------------------------------------
function adsm_able {

  echo "INFO: entering adsm_able " | tee -a $log
  export adsm_down="false"

  echo "$server_list = server_list" | tee -a $log
  for server in $server_list
  do
      export server=$server
      which_server $server
      admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
      if [ "$admInf" = '' ]
      then
         admin_id $server
      fi
      export admInf=$admInf
      echo "INFO: checking to see if server $server is up." | tee -a $log
      dsmadmc $admInf quit >> /dev/null 2>>/dev/null
      rc=$?
      if [ "$rc" != '0' ]
      then
         let cont=0
      else
         echo "INFO: server $server is up." | tee -a $log
         let cont=1
      fi
      while [ "$cont" = '0' ]
      do
         echo "INFO: could not communicate with ADSM server." | tee -a $log
         echo "INFO: will try to restart ADSM server." | tee -a $log
         startadsm $server >> /dev/null 2>> /dev/null
         #startadsm all &
         let loopCnt=10
         while [ "$loopCnt" -gt '0' ]
         do
            echo "INFO: will sleep for 1 minute to let it come all the way up." | tee -a $log
            sleep 30
            echo "INFO: issuing 'isitup' to see if ADSM server is up. " | tee -a $log
            isitup $server
            rc=$?
            if [ "$rc" != '0' ]
            then
               echo "WARNING: ADSM server $server did not come up.  Assuming it has SERIOUS problems" | tee -a $log
               export adsm_down="true"
               let cont=1
               let loopCnt=0
            else
               dsmadmc $admInf quit >> /dev/null 2>>/dev/null
               rc=$?
               if [ "$rc" != '0' ]
               then
                  echo "dsmadmc $admInf quit not coming back correctly yet." | tee -a $log
                  let loopCnt=loopCnt-1
               else
                  let loopCnt=0
                  let cont=1
              fi
            fi
         done
         if [ "$cont" = '0' ]
         then
            echo "WARNING: Can not communicate with ADSM server $server, please verify admin info. " | tee -a $log
            old_admin=$admInf
            export VERIFY="YES"
            admin_id $server
            export VERIFY="NO"
            if [ "$old_admin" = "$admInf" ]
            then
               echo "WARNING: can not communicate with ADSM server $server.  Assuming it has SERIOUS problems." | tee -a $log
               export adsm_down="true"
               let cont=1
            else
               dsmadmc $admInf quit >> /dev/null 2>>/dev/null
               rc=$?
               if [ "$rc" != '0' ]
               then
                  let cont=0
               else
                  let cont=1
               fi
            fi
         fi
      done
      thr=$(cat $rec_log | grep adsm_down | awk '{ print $2 }' )
      if [ "$thr" != "$adsm_down" -a "$thr" != '' ]
      then
         echo "INFO: updating $rec_log info on adsm_down." | tee -a $log
         realFile=$rec_log
         workFile=$rec_log.temp
         #sed -e "/adsm_down /d" $realFile > $workFile
         sed -e "s/adsm_down / /" < $realFile > $workFile
         cmd="cp $workFile $realFile"
         echo "Issuing: $cmd" | tee -a $log
         $cmd
         rm $workFile
         echo "adsm_down $adsm_down" >> $rec_log
      elif [ "$thr" = "$adsm_down" ]
      then
         echo "INFO: adsm_down already in $rec_log." | tee -a $log
      else
         echo "adsm_down $adsm_down" >> $rec_log
      fi
  done
  echo "INFO: leaving adsm_able " | tee -a $log
}


# --------------------------------------------------------------------
# adsm_lvl:  Obtain level of ASDM server. Verify it is v3 too.
# --------------------------------------------------------------------
function adsm_lvl {
  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"ADSM Server " $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi

  echo "INFO: entering adsm_lvl " | tee -a $log

  # looking at adsm.devices.rte because server.rte does not get uninstalled

  echo "INFO: obtaining level of ADSM code loaded." | tee -a $log
  whats_installed=$(lslpp -L all | grep adsm.devices.rte)
  tivoli=$(lslpp -l | grep -i "tivoli.tsm.server.com")
  if [ "$tivoli" = '' -a "$whats_installed" != '' ]
  then
     vrmf=$(echo $whats_installed | awk ' {print $2 } ' )
     echo ${vrmf} | sed -e "s/\./ /g" | read VER REL MOD FIX
     echo "INFO: ADSM LEVEL: $vrmf" | tee -a $log
   
     if [ "$VER" != '3' ]
     then
        echo "ERROR:  This is not a version 3 ADSM server.  It is a $VER.$REL.$MOD.$FIX." | tee -a $log
        if [ "$VER" = '2' ]
        then
           echo "   During prepare.ec phase 3 version 2 db will be updated." | tee -a $log
           echo "V2OLDADSMLEVEL" >> $rec_log
        fi
     else
        echo "INFO: This is a version 3 ADSM server." | tee -a $log
        if [ "$MOD" -lt '3' ]
        then
          echo "INFO: This will have to have upgrade db run against it." | tee -a $log
          echo "V31OLDADSMLEVEL" >> $rec_log
        fi
     fi

     echo "ADSM Server  $vrmf" >> $rec_log
  elif [ "$tivoli" != '' -a "$whats_installed" = '' ]
  then
     echo "INFO: Tivol Storaage Manager server already installed." | tee -a $log
     echo "V37TIVOLISTORAGEMANGER" >> $rec_log
  else  
     echo "ERROR:  lslpp -l is showing both and ADSM server and a Tivoli Storage Manager" | tee -a $log
     echo "$tivoli $whats_installed" | tee -a $log
  fi
 
  echo "INFO: leaving adsm_lvl " | tee -a $log
}

# --------------------------------------------------------------------
# adsmdown:  Verify server is down.  Stop if not (invoke endadsm)
# --------------------------------------------------------------------
function adsmdown {

   echo "INFO: entering adsmdown "  | tee -a $log
   for SERV in $server_list
   do
      which_server $SERV
      admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
      if [ "$admInf" = '' ]
      then
         admin_id $server
      fi
      export admInf=$admInf
      lock=adsmserv.lock
      serverup="false"

      if [ "$SERV" = "primary" ]
      then
          tivoli=$(lslpp -l | grep -i "tivoli.tsm.server.com")
          if [ "$tivoli" = '' ]
          then
             export servdir="/usr/lpp/adsmserv/bin"
          else
             export servdir="/usr/tivoli/tsm/server/bin"
          fi
      else
          servdir="/$SERV"
      fi
      if [ -f $servdir/$lock ]
      then
        pid=$(cat $servdir/$lock | awk '{ print $4 }' )
        pidthr=$(ps -eaf | grep -v grep | grep "$pid " )
        if [ "$pidthr" != '' ]
        then
           serverup="true"
           echo "INFO: Server $SERV is up."
        else
           kill -9 $pid
           serverup="false"
           echo "INFO: Server $SERV is not up."
        fi
      else
        echo "INFO: Server $SERV is currently halted."
        serverup="false"
      fi

      if [ "$serverup" = "true" ]
      then
         let tries=20
         while [ "$tries" -gt '0' ]
         do
            admInf=$(cat $rec_log | grep ADMINF$SERV | sed -e "s/ADMINF$SERV/ /" )
            dsmadmc $admInf halt >> /dev/null 2>/dev/null
            rc=$?
            if [ rc -gt 0 -a "$tries" = '1' ]
            then
               echo "ERROR: Could not halt the ADSM server $SERV." | tee -a $log
               echo "    Issued: dsmadmc $admInf halt" | tee -a $l
               echo "Please halt the ADSM server $SERV.  Press <ENTER> when this is done." | tee -a $log
               read
               let tries=0
            elif [ rc -gt 0 ]
            then
               echo "Sleeping while server comes up." | tee -a $log
               sleep 30
               let tries=tries-1
            else
               echo "SUCCESS: ADSM server $SERV halted." | tee -a $log
               let tries=0
            fi
         done
      else
         echo "ADSM server $SERV appears to be already halted." | tee -a $log
      fi
   done


   echo "INFO: leaving adsmdown " | tee -a $log
}

# --------------------------------------------------------------------
# function apply_updates: install all the fixes being added on
# --------------------------------------------------------------------
function apply_adsm_updates {
   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "APPLIED_adsm_UPDATES " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
  echo "INFO: entering apply_adsm_updates " | tee -a $log

  # --------------------------------------------------------------
  # Build the installp command
  # --------------------------------------------------------------

  INSTALLP="installp -a -V 2 -d ${updateADSMLoc} -f $installADSMList -c -N -g -X"
  echo
  print "INFO: Installing ADSM server software at latest level from install list." | tee -a $log
  print "      This will take from 5 to 20 minutes to complete." | tee -a $log
  print "      (depending on what is already installed on your 3466)"
  echo "       $INSTALLP" | tee -a $log

  ${INSTALLP} >> $log 2>&1
  installp_rc=$?


  # ---------------------------------------------------------------------
  # Verify ec installed ok in case rc not 0
  # ---------------------------------------------------------------------
  if [ $installp_rc != 0 ]
  then
     let junk=0
     while [ "$junk" != '1' ]
     do
        echo
        echo  "**************************************************************************"
        print "INFO:   Some filesets did not install." | tee -a $log
        echo  "         Check the $log for more information "
        echo  "         on what did not install."
        let junk=1
     done
  else
    print "SUCCESS: Filesets ADSM installed." | tee -a $log

  fi


  echo "INFO: leaving apply_adsm_updates " | tee -a $log
}


# --------------------------------------------------------------------
# function apply_updates: install all the fixes being added on 
# --------------------------------------------------------------------
function apply_updates {
   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "APPLIED_UPDATES " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
  echo "INFO: entering apply_updates " | tee -a $log

  # --------------------------------------------------------------
  # Build the installp command
  # --------------------------------------------------------------
  
  INSTALLP="installp -a -V 2 -d ${updateImageLoc} -f $installList -c -N -g -X"
  echo
  print "INFO: Installing software at latest level from install list." | tee -a $log
  print "      This will take from 5 to 55 minutes to complete." | tee -a $log
  print "      (depending on what is already installed on your 3466)"
  echo "       $INSTALLP" | tee -a $log
  
  ${INSTALLP} >> $log 2>&1
  installp_rc=$?


  # ---------------------------------------------------------------------
  # Verify ec installed ok in case rc not 0
  # ---------------------------------------------------------------------
  if [ $installp_rc != 0 ]
  then
     let junk=0
     while [ "$junk" != '1' ]
     do
        echo
        echo  "**************************************************************************"
        print "INFO:   Some filesets did not install." | tee -a $log
        echo  "         Check the $log for more information "
        echo  "         on what did not install."
        let junk=1 
     done
  else
    print "SUCCESS: Filesets installed." | tee -a $log
  
  fi


  echo "INFO: leaving apply_updates " | tee -a $log
}

#---------------------------------------------------------------------
# cleanup_adsm  must remove some stuff left out by tsm install
#---------------------------------------------------------------------
function cleanup_adsm {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"cleanup_adsm" $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi

  echo "INFO: entering cleanup_adsm" | tee -a $log
  thr=$( cat $rec_log | grep V37TIVOLISTORAGEMANGER )
  if [ "$thr" != '' ]
  then
    echo "INFO: don't need to deinstall server.rte as already TSM3.7" | tee -a $log
  else
     cmd="installp -ug adsm.server.rte"
     echo "INFO: issuing $cmd" | tee -a $log
     $cmd
  fi

  echo "cleanup_adsm" >> $rec_log
  echo "INFO: leaving cleanup_adsm" | tee -a $log

}


# --------------------------------------------------------------------
# copy_in_files 
# --------------------------------------------------------------------
function copy_in_files {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"copy_in_files " $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi

  echo "INFO: entering copy_in_files" | tee -a $log


  # only copy in SWA on h50.  r20 and h10 are too small
  if [ "$processorType" = "H50" ]
  then
     echo
     echo "Copying images for the secure TSM web admin client"
     sourceDir="$microDir/$tsmSWAdir"
     targetDir="$targetCopyDir/$tsmSWAdir"
     if [ -d $targetDir ]
     then
       echo
       echo "The target directory: $targetDir already exists"
     else
       theCommand="mkdir -p $targetDir"
       echo
       echo "Issuing command: $theCommand"
       eval $theCommand
       mkdirRc=$?
       if [ $mkdirRc != 0 ]
       then
         echo
         echo "ERROR: error making target directory for the secure TSM web admin client"
         let maxRC=1 # set to reveal failure
       fi
     fi
  else
    echo "INFO: not copying in tsmSWA as not enough space." | tee -a $log
  fi

  #-----------------------------------------------------
  #  install the web stuff
  #-----------------------------------------------------
  imageLoc="$microDir/webstuff"
  ilist=${imageLoc}/install.list
   
  INSTALLP="installp -qacgvNX -V 2 -d ${imageLoc} -f $ilist"
  echo | tee -a $log
  echo "INFO: installing web client filesets from $imageLoc"| tee -a $log
   
  print "INFO:    Installing software at latest level from install list." | tee -a $log
  echo "     $INSTALLP" | tee -a $log
   
  ${INSTALLP} >> $log 2>&1

  if [ ! -d $targetCopyDir/$tsmDOCdir   ]
  then
   
     # -----------------------------------------------------
     #  remove the old documents
     # -----------------------------------------------------
     if [ -d $ADSMDOCdir ]
     then
        echo "INFO: removing old ADSM documentation in $ADSMDOCdir" | tee -a $log
        rm -r $ADSMDOCdir
     fi
   
     # -----------------------------------------------------
     # copy the TSM AIX html documentation"
     # -----------------------------------------------------
     echo
     echo "Copying images for the TSM AIX html documentation"
     sourceDir="$microDir/$tsmDOCdir"
     targetDir="$targetCopyDir/$tsmDOCdir"
     if [ -d $targetDir ]
     then
       echo
       echo "The target directory: $targetDir already exists"
     else
       theCommand="mkdir -p $targetDir"
       echo
       echo "Issuing command: $theCommand"
       eval $theCommand
       mkdirRc=$?
       if [ $mkdirRc != 0 ]
       then
         echo
         echo "ERROR: error making target directory for the TSM AIX html documentation"
         let maxRC=1 # set to reveal failure
       fi
     fi
     
     theCommand='cp -R '"$sourceDir/"'*'" $targetDir"
     echo
     echo "Issuing command: $theCommand"
     eval $theCommand
     cpRc=$?
     if [ $cpRc != 0 ]
     then
       echo
       echo "ERROR: error copying secure TSM AIX html documentation"
       let maxRC=1 # set to reveal failure
     fi
     
     
     # --------------------------------------------------
     # set symbolic link for documentation
     # --------------------------------------------------
     theCommand="rm $tsmDocLink"
     echo
     echo "Remove symbolic link in case it already exists"
     echo "Issuing command: $theCommand"
     eval $theCommand
     
     theCommand="ln -sf $targetDir $tsmDocLink"
     echo
     echo "Inititalize symbolic link"
     echo "Issuing command: $theCommand"
     eval $theCommand
     linkRc=$?
     linkRc=$?
     if [ $linkRc != 0 ]
     then
       echo
       echo "ERROR: error initializing the symbolic link"
       let maxRC=1 # set to reveal failure
     else
       if [ -L $tsmDocLink/adsm.htm ]
       then
         echo
         echo "Symbolic link already established for $tsmDocLink/adsm.htm"
       else
         theCommand="cd $tsmDocLink"
         echo
         echo "Issuing command: $theCommand"
         eval $theCommand
         commandRC=$?
         if [ $commandRC != 0 ]
         then
           let maxRC=1
     
         else
           theCommand='ln -s index.htm adsm.htm'
           echo
           echo "Issuing command: $theCommand"
           eval $theCommand
           commandRC=$?
           if [ $commandRC != 0 ]
           then
             let maxRC=1
           fi
     
           theCommand="cd -"
           echo
           echo "Issuing command: $theCommand"
           eval $theCommand
         fi
       fi
     fi
  fi

  if [ "$maxRC" = '1' ]
  then
    echo "ERROR:  could not copy in the TSM files correctly."
    exit 1
  fi

  echo "copy_in_files " >> $rec_log
  echo "INFO: leaving copy_in_files" | tee -a $log

}

# --------------------------------------------------------------------
# call_fixSSA
# --------------------------------------------------------------------
function call_fixSSA {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"FIXSSADONE" $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi

  echo "INFO: entering call_fixSSA" | tee -a $log

  fixSSA

  echo "FIXSSADONE" >> $rec_log
 
  echo "INFO: leaving call_fixSSA" | tee -a $log
}



# --------------------------------------------------------------------
# check_ec
# --------------------------------------------------------------------
function check_ec {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"CHECK_EC" $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi


  str=$(lslpp -l all | grep wcm.rte)
  if [ "$str" != '' ] 
  then
    echo "ERROR:  This is a WCM machine and this EC can not be applied."
    exit 0
  fi
  lvl2=$(lsSSlevel | grep $ECLEVEL2) 
  if [ "$lvl1" != '' -o "$lvl2" != '' ]
  then
     echo "INFO: $lvl1 $lvl2 "  | tee -a $log
     echo "      You do not need to install this EC. " | tee -a $log
     exit 0
  else
     lsSSlevel | tee -a $log
  fi

  prereqlvl=$(lsSSlevel | grep "applied on" | grep $prereqEC)
  prereqlvl2=$(lsSSlevel | grep "applied on" |  grep $prereqEC2)
  prereqlvl3=$(lsSSlevel |  grep "applied on" | grep $prereqEC3)
  prereqlvl4=$(lsSSlevel |  grep "applied on" | grep $prereqEC4)
  if [ "$prereqlvl" = '' -a "$prereqlvl2" = '' -a "$prereqlvl3" = '' -a "$prereqlvl4" = '' ]
  then
    echo
    echo "***************************************************************************************"
    echo "ERROR: you don't have one of the prereq ECs $prereqEC $prereqEC2 $prereqEC3 installed."
    echo "  This must be installed before $ECLEVEL can be installed."
    exit 1
  fi

  lvl=$(oslevel)
  #lvl="JUNK"   # force this, as we do not care about oslevel for this ec
  if [ "$lvl" != $OSLEVEL ]
  then
     echo
     echo "***********************************************************" | tee -a $log
     echo "WARNING: $lvl is OS currently applied to this machine." | tee -a $log
     echo "   We only expect to see OS level $OSLEVEL applied. " | tee -a $log
     echo "   We have not tested this ec on other OS levels." | tee -a $log
     echo "   You may hit unexpected results on this install. " | tee -a $log
     let cont=0
     while [ "$cont" = '0' ]
     do
       echo "  1 )  continue despite AIX $lvl being applied. " | tee -a $log
       echo "  2 )  EXIT this script.  Don't apply EC. "
       read cont
       case $cont in
       1 ) echo "Continuing";;
       2 ) exit 0;;
       * ) let cont=0
       esac
     done
  else
     echo "------------------------------------------------------"
     echo "INFO: $lvl AIX is applied to this machine. "  | tee -a $log
  fi
  echo "INFO  The following ADSM server levels are applied to the machine."
  lslpp -l  adsm.server.rte | tee -a $log

  echo "CHECK_EC" >> $rec_log
  echo "INFO:  leaving check_ec " | tee -a $log
}


# --------------------------------------------------------------------
# check_3494
# --------------------------------------------------------------------
function check_3494 {

   echo "INFO: entering check_3494 " | tee -a $log

   Command="mtlib -l /dev/lmcp0 -qL"
   echo "ISSUING: $Command" | tee -a $log
   $Command
   rc=$?
   if [ "$rc" != '0' ]
   then
     echo "INFO: failure in mtlib" | tee -a $log
     aMsg="FAILURE:  Can not communicate with library.  Check hardware."
     echo $aMsg | tee -a $log
     exit 1
   else
     aMsg="SUCCESS: Communicated successfully with library /dev/lmcp0."
     echo "$aMsg" | tee -a $log
   fi
   cmd="mtlib -l /dev/lmcp0 -f /dev/rmt0 -qD"
   echo "ISSUING: $cmd" | tee -a $log
   mtl0=$(mtlib -l /dev/lmcp0 -f /dev/rmt0 -qD | grep "Device not available" )
   if [ "$mtl0" !=  '' ]
   then
      aMsg="FAILURE: Tape drive rmt0 is not available to the library. "
      echo "$aMsg" | tee -a $log
      aMsg="   At the library console, makesure the drive is available to the library. "
      echo "$aMsg" | tee -a $log
      exit 1 
   else
      echo "INFO: able to communicate with /dev/rmt0" | tee -a $log
   fi 

   echo "INFO: leaving check_3494 " | tee -a $log
}

# --------------------------------------------------------------------
# check_3575
# --------------------------------------------------------------------
function check_3575 {

   echo "INFO: entering check_3575 " | tee -a $log
   str=$(tapeutil -f/dev/smc0 inquiry | grep failed)
   if [ "$str" != '' ]
   then
      echo "ERROR: tapeutil can't talk with library smc0" | tee -a $log
      return 1
   else
      echo "SUCCESS: tapeutil inquiry worked for library smc0" | tee -a $log
   fi
   # --------------------------------------------------------------------
   # check that tapeutil can issue an inquiry against the drives
   # --------------------------------------------------------------------
   rmt="rmt0"
   echo "INFO: Checking tapeutil inquiry against drive" | tee -a $log
   str=$(tapeutil -f/dev/$rmt inquiry | grep failed)
   if [ "$str" != '' ]
   then
     echo "ERROR: tapeutil can't talk with drive  $rmt" | tee -a $log
     return 1
   else
     echo "INFO: tapeutil can talk with drive $rmt" | tee -a $log
   fi

   echo "INFO: leaving check_3575 " | tee -a $log
}

# --------------------------------------------------------------------
# check_prega_ssa
# --------------------------------------------------------------------
function check_prega_ssa {

# update for correct level on correct type of ssa

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"SSA_CHECKED_FORPREGA" $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   echo "INFO: entering check_prega_ssa " | tee -a $log
   downLevelFound="no"
   # --------------------------------------------------------------------
   # Check for really bad levels of microcode
   # --------------------------------------------------------------------
   str=$(lscfg -vl pdisk* | grep "ROS Level" | grep 45)
   if [ "$str" != '' ]
   then
      echo "WARNING: Down level microcode (45)  on disk. Will update."
      downLevelFound="yes"
   fi
   str=$(lscfg -vl pdisk* | grep "ROS Level" | grep 46)
   if [ "$str" != '' ]
   then
      echo "WARNING: Down level microcode (46)  on disk. Will update."
      downLevelFound="yes"
   fi
   str=$(lscfg -vl pdisk* | grep "ROS Level" | grep 47)
   if [ "$str" != '' ]
   then
      echo "WARNING: Down level microcode (47)  on disk. Will update."
      downLevelFound="yes"
   fi

   if [ "$downLevelFound" = "yes" ] 
   then

     echo "Moving dasd download packages out of /etc/microcode..."
     mkdir /tmp/disk45 2>/dev/null
     if [[ ! -d /tmp/disk45 ]]
     then
       echo "problem creating /tmp/disk45 - aborting..."
       exit 1
     fi

     mv /etc/microcode/ssadisk.ros.DGHC* /tmp/disk45 2>/dev/null
     mv /etc/microcode/ssadisk.ros.WAAH* /tmp/disk45 2>/dev/null

     echo "Moving level 46 microcode to /etc/microcode..."
     cp $microDir/ssadisk.ros.DGHC.0046 /etc/microcode
     if [[ $? -ne 0 ]]
     then
        echo "problem with ssadisk.ros.DGHC.0046 to /etc/microcode - aborting"
        TidyUp
        exit1
     fi

     echo "Downloading level 46..."
     ssadload -u
     if [[ $? -ne 0 ]]
     then
        echo "ERROR:  Could not install level 46 for SSA microcode."
        echo "        Contact support."
        exit 1
     fi

     echo "Removing level 46 code from /etc/microcode..."
     rm /etc/microcode/ssadisk.ros.DGHC.0046

     echo "Moving level 47 dasd download code to /etc/microcode"
     cp $microDir/ssadisk.ros.DGHC.0047 /etc/microcode
     if [[ $? -ne 0 ]]
     then
        echo "ERROR:  Could not install level 47 for SSA microcode."
        echo "        Contact support."
        TidyUp
        exit1
     fi
     echo "Downloading level 47..."
     ssadload -u
     rc=$?
     echo "Removing level 47 code from /etc/microcode..."
     rm /etc/microcode/ssadisk.ros.DGHC.0047

     echo "Moving level 49 dasd download code to /etc/microcode"
     cp $microDir/ssadisk.ros.DGHC.0049 /etc/microcode
     if [[ $? -ne 0 ]]
     then
        echo "ERROR:  Could not install level 49 for SSA microcode."
        echo "        Contact support."
        TidyUp
        exit1
     fi
     echo "Downloading level 49..."
     ssadload -u
     rc=$?
     echo "Removing level 49 code from /etc/microcode..."
     rm /etc/microcode/ssadisk.ros.DGHC.0049

     if [[ $rc -ne 0 ]]
     then
        echo "problem downloading level 49......"
     else
        echo "Level 49 has been downloaded successfully to all drives..."
     fi

   fi
   echo "SSA_CHECKED_FORPREGA" >> $rec_log
   echo "INFO: leaving check_prega_ssa " | tee -a $log

}

# --------------------------------------------------------------------
# check_ssa
# --------------------------------------------------------------------
function check_ssa {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"SSAOK" $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi
   echo "INFO: entering check_ssa " | tee -a $log
   str=$(lsdev -Cc disk | grep defined)
   str2=$(lsdev -Cc disk | grep Defined)
   if [ "$str" != '' -o "$str2" != '' ] 
   then
      echo "ERROR:  There is a problem with an SSA hdisk."
      echo "        Correct problem and then restart this script."
      exit 1
   fi
   str=$(lsdev -Cc pdisk | grep defined)
   str2=$(lsdev -Cc pdisk | grep Defined)
   if [ "$str" != '' -o "$str2" != '' ] 
   then
      echo "ERROR:  There is a problem with an SSA pdisk."
      echo "        Correct problem and then restart this script."
      exit 1
   fi

   # Verify varyonline is yes and quorum is yes always!
   echo "Info:  Updating qurom and vgonline for adsmvg" | tee -a $log
   chvg -a'y' -Q'y' -x'n' adsmvg
   count=$(lsdev -Cc disk | grep -c SSA)
   if [ "$count" -gt 32 ] 
   then
      echo "Info:  Updating qurom and vgonline for adsmvg2" | tee -a $log
      chvg -a'y' -Q'y' -x'n' adsmvg2
   fi

   echo "SSAOK" >> $rec_log
   echo "INFO: leaving check_ssa " | tee -a $log
}
# --------------------------------------------------------------------
# check_tape
# --------------------------------------------------------------------
function check_tape {

  echo "INFO: entering check_tape." | tee -a $log
  mod=$1
  echo 
  echo "***************************************************"
  echo "This is a model $mod." | tee -a $log
  echo "This script can not predict how the drives should look." | tee -a $log
  echo "The CUSTOMER needs to make sure that the drives in AIX " | tee -a $log
  echo "match the defintions in ADSM. " | tee -a $log
  echo "  At AIX issue:  lsdev -Cc tape " | tee -a $log
  echo "  IN ADSM issue: q library   q drive " | tee -a $log
  echo
  echo "Press <ENTER> to continue."  
  read
 

  echo "INFO: entering check_tape." | tee -a $log
}


#-------------------------------------------------------
#  deterimne the model of the old system.
#-------------------------------------------------------
function det_model {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"model is" $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   echo "INFO: entering det_model" | tee -a $log
   if [ -f "$loc3466/$model_file" ] 
   then 
      whatmodel | tee -a $log
      theModel="MODELALREADYTHERE"
   else
      clear
      print
      print "What model is this system? You MUST check MLC records and provide."
      print "an accurate answer. Do NOT rely simply on attached tape type."
      print
      PS3="Enter choice: "
      select theModel in \
         'B01 '\
         'B02 '\
         'B03 ' \
         'A00 ' \
         'A01 with FC 4206, 4208, 4218 (3575)' \
         'A01 with FC 4202 (DLT)'\
         'C00 '\
         'C10 '\
         'C20 '\
         'C30 '

      do
        case $REPLY in
          1) is3590=$(lsdev -Cc tape | grep 3590)
             if [ "$is3590" = '' ]
             then
                echo "ERROR:  You said this was a B model but no 3590 drives were found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;
          2) is3590=$(lsdev -Cc tape | grep 3590)
             if [ "$is3590" = '' ]
             then
                echo "ERROR:  You said this was a B model but no 3590 drives were found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;
          3) is3590=$(lsdev -Cc tape | grep 3590)
             if [ "$is3590" = '' ]
             then
                echo "ERROR:  You said this was a B model but no 3590 drives were found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;
          4) ;;
          5) is3575=$(lsdev -Cc tape | grep 3575)
             if [ "$is3575" = '' ]
             then
                echo "ERROR:  You said this was a A model with 3575 but no 3575 wasfound."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             theModel="A01 with 3575"
             ;;
          6) is7337=$(lsdev -Cc tape | grep DLT)
             if [ "$is7337" = '' ]
             then
                echo "ERROR:  You said this was a A model with DLT but no DLT was found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             theModel="A01 with 7337"
             ;;
          7) ;;
          8) is7337=$(lsdev -Cc tape | grep DLT)
             if [ "$is7337" = '' ]
             then
                echo "ERROR:  You said this was a C10 model with DLT but no DLT was found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;
          9) is3575=$(lsdev -Cc tape | grep 3575)
             if [ "$is3575" = '' ]
             then
                echo "ERROR:  You said this was a C20 model with 3575 but no 3575 wasfound."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;
          10) is3590=$(lsdev -Cc tape | grep 3590)
             if [ "$is3590" = '' ]
             then
                echo "ERROR:  You said this was a C30 model but no 3590 drives were found."
                echo "        Determine what model this is and restart this script."
                exit 1
             fi
             ;;

        *) print 'Invalid Selection.' ;;
        esac
        if [[ -n $theModel ]] ; then
           break
        fi

      done
    fi
    echo "model is $theModel" >> $rec_log                        
    echo "INFO: leaving det_model" | tee -a $log
}
# --------------------------------------------------------------------
# db_backup_thr  verify there is a dbbackup from today 
# --------------------------------------------------------------------
function db_backup_thr {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"orig_series " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
   for server in $server_list
   do
      which_server $server
      admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
      if [ "$admInf" = '' ]
      then
         admin_id $server
      fi
      export admInf=$admInf
      echo "INFO: entering db_backup_thr at $(date)" | tee -a $log
      #--------------------------------------------------------------
      # get series of last db backup
      #--------------------------------------------------------------
      dsmadmc $admInf q volhistory type=dbbackup begindate=today-1 > /tmp/hold
      num_bk=$(cat /tmp/hold | grep -c "BACKUPFULL")
      old=$(cat /tmp/hold | grep "\-\-\-\-\-\-\-\-")
      if [ "$old" != '' ]     ## two different outputs depending on client level
      then
         orig_series=$(cat /tmp/hold | grep "BACKUPFULL" | grep -n "BACKUPFULL" | grep ^$num_bk: | awk '{ print $4 }')
      else
         orig_series=$(cat /tmp/hold | grep "Backup Series:" | grep -n "Backup Series:" | grep ^$num_bk: | awk '{ print $4 }')
      fi
      if [ "$orig_series" = '' -a "$server" = "primary" ]
      then
         clear
         echo
         echo "***************************************************"
         echo "ERROR: Could not ascertain which volume was used for the" | tee -a $log
         echo "dbbackup of server $server" | tee -a $log
         echo "  It appears no database backup was done in last 24 hours." | tee -a $log
         echo "  Have the custmer create an ADSM database backup." | tee -a $log
         echo "  Then rerun this script." | tee -a $log
         exit 1
      elif [ "$orig_series" = ''  ]
      then
         echo  
         echo "WARNING: Could not ascertain which volume was used for the " | tee -a $log
         echo "dbbackup of server $server" | tee -a $log
         echo "  It appears no database backup was done in the last 24 hours."  | tee -a $log
         echo "  If one is not created, there is the possibility of lost data on this kin server." | tee -a $log
         echo "  Do you want to continue?"
         PS3="Select Option "
         select option in \
         '    EXIT this program and let customer do backup of this kin server. ' \
         '    CONTINUE - this kin server is not in use.  '
         do
            case $REPLY in
            1 ) exit 1
                break;;
            2 ) echo
                break;;
            * ) echo "INVALID selection";;
            esac
         done
      else 
         echo "orig_series$serv $orig_series" >> $rec_log
      fi
   done
   echo "INFO: leaving db_backup_thr at $(date)" | tee -a $log
}

# --------------------------------------------------------------------
# def_3494_lib: define the 3494 library 
# --------------------------------------------------------------------
function def_3494_lib {
 
   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "def_3494_lib " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi


  echo "INFO: entering def_3494_lib " | tee -a $log
  libStr=`lsdev -C | grep lmcp0`
  if [ "$libStr" = '' ]
  then
      aixCommand="eval /etc/methods/defatl -ctape -slibrary -tatl -l'lmcp0' -a library_name='3494a'"
      echo "ISSUING: $aixCommand" | tee -a $log
      $aixCommand
      aixrc=$?
      if [ "$aixrc" != 0 ]
      then
         echo "FAILURE:  $aixrc from $aixCommand."
         exit 1
      fi
   
      aixCommand="mkdev -l lmcp0"
      echo "ISSUING: $aixCommand" | tee -a $log
      $aixCommand
      aixrc=$?
      if [ "$aixrc" != 0 ]
      then
         echo "FAILURE:  $aixrc from $aixCommand."
         exit 1
      fi
   fi
   
   # --------------------------------------------------------------------
   # make a tty for the 3494 library (the tty port must resolve to tty1)
   # --------------------------------------------------------------------
   ttyStr=`lsdev -C | grep tty1`
   if [ "$ttyStr" = '' ]
   then
      aixCommand="eval mkdev -c tty  -t 'tty' -s 'rs232' -p 'sa1' -w 's2'"
      echo "ISSUING: $aixCommand" | tee -a $log
      $aixCommand
      aixrc=$?
      if [ "$aixrc" != 0 ]
      then
         echo "FAILURE:  $aixrc from $aixCommand."
         exit 1
      fi
   fi
  
  echo "def_3494_lib " >> $rec_log
  echo "INFO: leaving def_3494_lib " | tee -a $log

}


# --------------------------------------------------------------------
# ec_stamp_unit:  stamp unit with ec number, date, time , and lslpp
# --------------------------------------------------------------------
function ec_stamp_unit {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "EC_STAMPED_ON_UNIT" $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

  echo "INFO: entering ec_stamp_unit " | tee -a $log
  InstallDate=`date +"%m%d%Y"`
  InstallTime=`date +"%H%M"`

  ECFILENAME=WebShlLvl.$InstallDate$InstallTime

  printf "$ECLEVEL: Filesets installed for EC level \n" > /var/adm/$ECFILENAME
  printf "Date installed is $InstallDate\n" >> /var/adm/$ECFILENAME
  lslpp -L all >> /var/adm/$ECFILENAME


  echo "EC_STAMPED_ON_UNIT " >> $rec_log
  echo "INFO: leaving ec_stamp_unit " | tee -a $log
}

# --------------------------------------------------------------------
# end_adsm:  Halt the server via endadsm.
# --------------------------------------------------------------------
function end_adsm {
  which_serv=$1
  echo "INFO: entering end_adsm " | tee -a $log
  echo "INFO: calling endadsm." | tee -a $log
  endadsm $which_serv
  echo "INFO: leaving end_adsm " | tee -a $log
}


# --------------------------------------------------------------------
# end_phase_1:  This is the end of the script for phase1.
# --------------------------------------------------------------------
function end_phase_1 {

   echo "INFO: entering end_phase_1 " | tee -a $log
   echo "This is the end of phase 1." | tee -a $log
   mv $rec_log $rec_log.phase1
   echo "INFO: leaving end_phase_1 " | tee -a $log
}

# --------------------------------------------------------------------
# end_it:  This is the end of the script.
# --------------------------------------------------------------------
function end_it {

   echo "INFO: entering end_it " | tee -a $log

   clear
   if [ "$man_pool" = '1' ]
   then
      echo
      echo "************************************************************"
      echo "INFO: There were some procedures you need to do manually." | tee -a $log
      echo "   The following list is in the $todo file:" | tee -a $log
      echo
      cat $todo
      echo "   Press <ENTER> to continue"
      read ;
   fi
   echo "SUCCESS: Phase 2 is complete. " | tee -a  $log
   echo "         Press  <ENTER> to acknowledge."
   read
   
}


# --------------------------------------------------------------------
# full_fs: make sure none of the rootvg fs are full
# --------------------------------------------------------------------
function full_fs {

  echo "INFO: entering full_fs " | tee -a $log
  rootvg="/ /var /tmp /usr /home"
  for fs in $rootvg
  do
     full=$(df $fs | grep $fs | awk '{ print $4 }' )
     if [ "$full" = "100%" ]
     then
       echo "ERROR: filesystem $fs is 100% full" | tee -a $log
       echo "    Delete unneeded info or expand $fs size." | tee -a $log
       echo "    Then rerun this program." | tee -a $log
       exit 1
     else
       echo "INFO: filesytem $fs is $full full." | tee -a $log
     fi
   done

  echo "INFO: leaving full_fs " | tee -a $log
}


# --------------------------------------------------------------------
# get_type_machine: find out if h10, h50 or r20
# --------------------------------------------------------------------
function get_type_machine {

  echo "INFO: entering get_type_machine " | tee -a $log
  # ----------------------------------------------------------
  # What type of Processor?  (H10, H50, R20?)
  # ----------------------------------------------------------
  str=$(uname -m | cut -c 9,10 )
  if [ "$str" = "4C" ]
  then
     str=$(lsattr -El sys0 | grep modelname | grep H50)
     if [ "$str" = '' ]
     then
        export processorType="H10"
     else
        export processorType="H50"
     fi
  elif [ "$str" = "72" ]
  then
    export processorType="R20"
  else
     echo "FAILURE: Processor type is not H10, H50, or R20"
     exit 1
  fi                    
  echo "TYPE_MACH $processorType" >> $rec_log 
  echo "INFO: leaving get_type_machine " | tee -a $log
}


# --------------------------------------------------------------------
# halt_adsm:  Halt the server via halt.
# --------------------------------------------------------------------
function halt_adsm {
   echo "INFO: entering halt_adsm " | tee -a $log
   for SERV in $server_list
   do
      which_server $SERV
      admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
      if [ "$admInf" = '' ]
      then
         admin_id $server
      fi
      export admInf=$admInf
      lock=adsmserv.lock
      serverup="false"

      if [ "$SERV" = "primary" ]
      then
          tivoli=$(lslpp -l | grep -i "tivoli.tsm.server.com")
          if [ "$tivoli" = '' ]
          then
             export servdir="/usr/lpp/adsmserv/bin"
          else
             export servdir="/usr/tivoli/tsm/server/bin"
          fi
      else
          servdir="/$SERV"
      fi
      sleep 10
      if [ -f $servdir/$lock ]
      then
        echo "INFO: $servdir/$lock exists." | tee -a $log
        pid=$(cat $servdir/$lock | awk '{ print $4 }' )
        pidthr=$(ps -eaf | grep -v grep | grep "$pid " )
        if [ "$pidthr" != '' ]
        then
           serverup="true"
           echo "INFO: Server $SERV is up."
        else
           kill -9 $pid
           serverup="false"
           echo "INFO: Server $SERV is not up."
        fi
      else
        echo "INFO: Server $SERV is currently halted."
        serverup="false"
      fi

      if [ "$serverup" = "true" ]
      then
         let tries=20
         while [ "$tries" -gt '0' ]
         do
            admInf=$(cat $rec_log | grep ADMINF$SERV | sed -e "s/ADMINF$SERV/ /" )
            dsmadmc $admInf halt >> /dev/null 2>/dev/null
            rc=$?
            if [ rc -gt 0 -a "$tries" -eq '1' ]
            then
               echo "ERROR: Could not halt the ADSM server $SERV." | tee -a $log
               echo "    Issued: dsmadmc $admInf halt" | tee -a $l
               echo "Please halt the ADSM server $SERV.  Press <ENTER> when this is done." | tee -a $log
               read
               let tries=0
            elif [ rc -gt 0 ]
            then
               echo "INFO: sleeping for 30 seconds while halting server." | tee -a $log
               echo "    using dsmadmc $admInf halt." | tee -a $log
               sleep 30
               let tries=tries-1
         
            else
               echo "SUCCESS: ADSM server $SERV halted." | tee -a $log
               let tries=0
            fi
         done
      else
         echo "ADSM server $SERV appears to be already halted." | tee -a $log
      fi
   done

  echo "INFO: leaving halt_adsm " | tee -a $log
}

#---------------------------------------------------------------------
# hsm_there:  If hsm is running on this machine - we may need to do
#             something
#---------------------------------------------------------------------
function hsm_there {
   echo "INFO: entering hsm_there" | tee -a $log
   thr1=$(cat /etc/adsm/SpaceMan/config/dsmmigfstab | grep -v \# ) 
   thr2=$(ps -eaf | grep -i dsmmonitord | grep -v grep | awk '{ print $2 }' )
   thr3=$(ps -eaf | grep -i dsmrecalld | grep -v grep | awk '{ print $2 }' )
   echo "INFO: thr1=$thr1 thr2=$thr2 thr3=$thr3" | tee -a $log

   if [ "$thr1" != '' -a "$thr2" != '' -a "$thr3" != '' ]
   then
      clear
      echo "***********************************************************************"
      echo "WARNING:  it appears ADSM heirarchical Storage Manager maybe installed." | tee -a $log
      echo "    The file /etc/adsm/SpaceMan/config/dsmmigfstab has enteries in it." | tee -a $log
      echo "    Tivoli Storage Manger client 3.7 currently does not support hsm."
      echo "    If HSM is installed the new client will not be installed"
      echo "    as this would cause  DATA WILL BE LOST." | tee -a $log
      echo
      PS3="Select Option: "
      select option in \
      '    YES, there is HSM installed.  The new level of Tivoli storage manager should not be installed. ' \
      '    NO, HSM is not installed, and the new client can be installed.'
      do
          case $REPLY in
          1 ) echo "INFO: not installing tsm client as hsm is present. "  | tee -a $log
              break;;
          2 ) install_client
              break;;
          * ) echo "INVALID selection";;
          esac
      done
   else
      echo "INFO: no heirarchical storage manager on this box." | tee -a $log
      install_client  
   fi
   echo "INFO: leaving hsm_there" | tee -a $log
}

# --------------------------------------------------------------------
# insert second cd
# --------------------------------------------------------------------
function insertSecondCd {
   clear
   let cont=0
   while [ "$cont" = '0' ]
   do
      echo "********************************************************"
      echo "PLEASE insert the second cd"
      echo "   - open another window."
      echo "   - make sure no one is in the /cd directory." 
      echo "   - issue 'ucdrom' to unmount the current CD."
      echo "   - remove the first CD. "
      echo "   - insert the second CD."
      echo "   - issue 'mcdrom to mount the second CD."
      echo 
      PS3="Select Option: "
      select option in \
      '    Continue, the second cd has been inserted ' \
      '    EXIT.'
      do
          case $REPLY in
          1 ) echo
              break;;
          2 ) exit 1;;
          * ) echo "INVALID selection";;
          esac
      done
  
      if [ -f $microDir/thisissecondcd ]
      then
          echo "The second cd has been mounted."
          let cont=1
      else
          echo "The second cd has not yet been mounted."
      fi
   done
 
}

# --------------------------------------------------------------------
# installNewAdsm:  Install fixtest server (3.1.2.13)
# --------------------------------------------------------------------
function installNewAdsm {
# pjj made need this code for something...currently not using
  echo "INFO: entering installNewAdsm" | tee -a $log

  str=$(lslpp -l all | grep adsm.server.rte | grep 3.1.2.13)
  if [ "$str" = '' ] 
  then

     INSTALLP="installp -qacgvNX -V 2 -d ${updateImageLoc} -f $installList"

     print "INFO:    Installing software at latest level from install list." | tee -a $log
     echo "    $INSTALLP" | tee -a $log

     ${INSTALLP} >> $log 2>&1
     installp_rc=$?
     if [ $installp_rc != 0 ]
     then
       print "ERROR: 3.1.2.1.1 did not install." | tee -a $log
       exit 1
     else
       print "SUCCESS: Filesets installed." | tee -a $log
     fi
  fi

  echo "INFO: leaving installNewAdsm" | tee -a $log
}

# --------------------------------------------------------------------
# function install adobe
# --------------------------------------------------------------------
function install_adobe {
  echo "INFO: entering install_adobe " | tee -a $log
   # -----------------------------------------------------------------------
   # determine if phase 1 of Adobe Acrobat is installed via AIX
   # installation inventory
   # -----------------------------------------------------------------------
   checkForAdobeInstall=`lslpp -l | grep Adobe.acrobat`
   if [ "$checkForAdobeInstall" = '' ]
   then
     echo
     echo "ERROR: Phase 1 of Adobe acrobat is not installed (lslpp -l | grep Adobe.acrobat)" | tee -a $log
     echo "       unable to continue, correct problem and re-invoke this routine" | tee -a $log
     echo
     theRc=1
   
   # -----------------------------------------------------------------------
   # determine if phase 2 of Adobe Acrobat is installed
   # -----------------------------------------------------------------------
   elif [ -d /usr/Adobe/acrobat ] # phase 2 of install is not complete
   then
      echo
      echo "INFO: implementing phase 2 of Adobe Acrobat installation" | tee -a $log
      cd /usr/Adobe/acrobat
      echo 'accept' | ./INSTALL > /tmp/acrobat3.install 2>&1 # install acrobat, accept license, and default installtion direcory
      echo
      cat /tmp/acrobat3.install
      rm /tmp/acrobat3.install >/dev/null 2>&1
      cd - >/dev/null 2>&1 # cd to previous directory
      if [ -f /usr/lpp/Acrobat3/bin/acroread ]
      then
        cd /usr/bin
        theCommand="ln -s /usr/lpp/Acrobat3/bin/acroread acroread"
        echo
        echo "INFO: issueing command: $theCommand" | tee -a $log
        eval $theCommand
        echo
        theCommand="rm -r /usr/Adobe" # remove source directory to indicate phase 2 of install is complete
        echo
        echo "INFO: issueing command: $theCommand" | tee -a $log
        eval $theCommand
        echo
        echo "INFO: phase 2 of Acobe Acrobat installation is complete" | tee -a $log
        echo
        theRc=0
      else
        echo
        echo "ERROR: phase 2 of install executed but /usr/lpp/Acrobat3/bin/acroread does not exist" | tee -a $log
        echo "       corrective action required" | tee -a $log
        echo
        theRc=2
      fi
   
   
   elif [ -f /usr/lpp/Acrobat3/bin/acroread ]
   then
     echo
     echo "INFO: Adobe Acrobat3 is already fully installed" | tee -a $log
     echo
     theRc=0
   else
     echo
     echo "ERROR: Adobe Acrobat3 program fully installed but the file" | tee -a $log
     echo "  /usr/lpp/Acrobat3/bin/acroread does not exist" | tee -a $log
     echo "  this condition is unexpected, contact IBM support" | tee -a $log
     echo
     theRc=3
   fi
   

  echo "INFO: leaving install_adobe " | tee -a $log

}
# --------------------------------------------------------------------
# function install client: install the client 
# --------------------------------------------------------------------
function install_client {
   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "install_client " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
  echo "INFO: entering install_client " | tee -a $log

  # --------------------------------------------------------------
  #  uninstall the hsm client
  # --------------------------------------------------------------
 
  echo "INFO: about to remove hsm client code" | tee -a $log
  cmd="installp -ug adsm.client.aix42.hsm"
  echo "INFO: issuing $cmd" | tee -a $log
  $cmd
  
  # --------------------------------------------------------------
  # Build the installp command
  # --------------------------------------------------------------
 
  ilist=${tsmclientdir}/install.list 
  INSTALLP="installp -qacgvNX -V 2 -d ${tsmclientdir} -f $ilist "
  echo
  print "INFO: Installing software at latest level from client install list." | tee -a $log
  print "      This will take around 5 minutes to complete." | tee -a $log
  echo "    $INSTALLP" | tee -a $log

  
  ${INSTALLP} >> $log 2>&1
  installp_rc=$?


  # ---------------------------------------------------------------------
  # Verify ec installed ok in case rc not 0
  # ---------------------------------------------------------------------
  if [ $installp_rc != 0 ]
  then
     let junk=0
     while [ "$junk" != '1' ]
     do
        echo
        echo  "**************************************************************************"
        print "INFO: Some filesets did not install." | tee -a $log
        echo  "         Check the $log for more" | tee -a $log
        echo  "         information on what did not install."
        let junk='1'
     done
  else
    print "SUCCESS: Filesets installed." | tee -a $log
    echo "INFO: creating link for MESs between old and new directories." | tee -a $log
    mv /usr/lpp/adsm/bin /usr/lpp/adsm/bin.old
    ln -s /usr/tivoli/tsm/client/ba/bin /usr/lpp/adsm/bin/
  
  fi


  echo "install_client " >> $rec_log
  echo "INFO: leaving install_client " | tee -a $log
}

# --------------------------------------------------------------------
# log_info:  Set up log files. Save old versions in date/time stamped
#            version.  If this is phase 2, let them start from where
#            they left off.  
# --------------------------------------------------------------------
function log_info {

   clear
   mp=`date +"%H%M%S"`
   if [ -f $rec_log ]
   then
      reRunScript="yes" 
   fi


   if [ "$phase" = "phase1" ]
   then
      export log=$dirForFiles/prepare1.log   # ongoing log of events
   elif [ "$phase" = "phase2" ]
   then
      export log=$dirForFiles/prepare2.log   # ongoing log of events
   else
      export log=$dirForFiles/prepare3.log   # ongoing log of events
   fi
   echo "This script will create the log $log." | tee -a $log

   # Determine if this is a machine with /usr/3466 or /usr/nsm.3466
   # for phase3 this will always be /usr/nsm.3466
   if [ "$phase" = "phase2" -o "$phase" = "phase1" ]
   then
      str=$(lslpp -l all | grep nsm.3466)
      if [ "$str" != ''  ]
      then
         export loc3466="/usr/nsm.3466"
      elif [ -d /usr/3466 ]
      then
         export loc3466="/usr/3466"
      else
         mkdir /usr/nsm.3466 >> /dev/null
         export loc3466="/usr/nsm.3466"
      fi

      if [ ! -d "$dirForFilesInAdsmlog" ] 
      then
         mkdir $dirForFilesInAdsmlog
      fi
   else
      if [ -d /usr/nsm.3466 ]
      then
         export loc3466="/usr/nsm.3466"
      else
         echo "ERROR:  Unexpected Problem - /usr/nsm.3466 does not exist." 
         echo "        Call support and report problem."
         exit 1
      fi
   fi

   thr=$(echo $PATH | grep "/usr/lpp/adsmserv/bin")
   if [ "$thr" = '' ]
   then
      echo "INFO: adding /usr/lpp/adsmserv/bin to the PATH." | tee -a $log
      export PATH=$PATH":/usr/lpp/adsmserv/bin"
   fi

  if [ ! -d /tmp/record ]
  then
    mkdir /tmp/record
  fi   

  
}


# --------------------------------------------------------------------
# mc_3575:  add in mc for drive and library 
# --------------------------------------------------------------------
function mc_3575 {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep "mc_3575 " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   echo "INFO: entering mc_3575" | tee -a $log

   # upgrade library microcode 
   is3570=$(lsdev -Cc tape | grep  "IBM Magstar MP" | grep -c -i tape)
   if [ "$is3570" -gt 0 ]
   then
      echo "INFO: Upgrading 3570 microcode for the $is3570 library." | tee -a $log
      echo "ATTENTION: You may need to update the 3575 firmware code." | tee -a $log
      echo "     Follow the Installation Instructions for the " | tee -a $log
      echo "     $MC3570 microcode update for library $drv." | tee -a $log
      echo
      let cont='0'
      while [ "$cont" != '1' ]
      do
         echo "TYPE '1' and press <enter> to continue."
         read cont
      done
      while [ "$is3570" -gt 0  ]
      do
         drv=$(lsdev -Cc tape | grep "IBM Magstar MP " | grep -i tape | grep -n -i tape  | grep $is3570: | tr : ' ' | awk '{ print $2 }')
         # --------------------------------------------------------------- 
         # Determine if this is a B or C model.
         # We assume that there is no mixed drives (ie no b drives in c libs)
         # --------------------------------------------------------------- 
         what_model $drv
         if [ "$mod" = "mod1" ]
         then
            MC3570=$MC3570B
            echo "INFO $drv is a MOD1 3575." | tee -a $log
         else
            MC3570=$MC3570C
            echo "INFO $drv is a MOD2 3575." | tee -a $log
         fi
         #tell them to update library manager code


         let is3570=is3570-1
   
 
         # upgrade drive  microcode 
         if [ "$drv" = "smc0" ]
         then  
            mod=$(lsdev -Cc tape | grep rmt0 | awk '{ print $3 }' | tr '-' ' ' | awk '{ print $1 }')
            if [ "$mod" = '20' ]  
            then
                moddrive="modc"   #20-60
                addr="20-60"
            else
                moddrive="modb"   #04-01 
                addr="04-01"
            fi
         else
            mod=$(lsdev -Cc tape | grep "10-68" | awk '{ print $3 }' | tr '-' ' ' | awk '{ print $1 }')
            if [ "$mod" != '' ]
            then
                moddrive="modc"   #10-68
                addr="10-68"
            else
                moddrive="modb"   #04-04
                addr="04-04"
            fi
         fi
         if [ "$moddrive" = "modc" ]
         then
            MC3575=$MC3575C
         else
            MC3575=$MC3575B
         fi

         let i=$(lsdev -Cc tape | grep $addr | grep -c "3570")
         while [ "$i" -gt '0' ]
         do
            rmt=$(lsdev -Cc tape | grep $addr | grep -C $addr | grep $i: | tr ':' ' ' | awk '{ print $2 }')
            if [ "$thr" != '' ]
            then
               need=$(tapeutil -f /dev/$rmt vpd | grep "Revision Level" | awk '{ print $4 }')
               if [ "$need" != $MC3570 ]
               then
                  echo "INFO: drive /dev/$rmt is at microcode level $need." | tee -a $log
                  echo "      will updated code to $MC3570" | tee -a $log
                  tapeutil -f/dev/$rmt ucode $microDir/$MC3570
               else
                  echo "INFO: /dev/$rmt already at level $need." | tee -a $log
               fi
            fi
            let i=i-1
         done
      done 
   fi 
   echo "mc_3575 " >> $rec_log
   echo "INFO: leaving mc_3575" | tee -a $log
}

# --------------------------------------------------------------------
# mc_7337:  add in mc for drive and library 
# --------------------------------------------------------------------
function mc_7337 {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"mc_7337 " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   cp $microDir/genucode.cat /usr/lpp/diagnostics/catalog/default
   cp $microDir/$MCdlt /tmp/
   rc=$?
   cp $microDir/$MCdltlowercase /tmp/$MCdlt
   cp $microDir/genucode /tmp/genucode
   cp $microDir/genucode. /tmp/genucode
   cp $microDir/dsdata /tmp/
   cp $microDir/dsdata. /tmp/
   echo "INFO: entering mc_7337" | tee -a $log

   # upgrade library  microcode 
   lvl="1.63"
   let i=1   #maximum of libraries is 2
   while [ "$i" -ge 0  ]
   do
      drv=$(lsdev -Cc tape | grep "IBM 7337 Tape Medium Changer" | grep "smc$i" | awk '{ print $1 }' ) 
      if [ "$drv" != '' ]
      then
         need=$(tapeutil -f /dev/$drv vpd | grep "Revision Level" | awk '{ print $4 }')
         if [ "$need" != $lvl ]
         then
            echo "INFO: Upgrading library $drv microcode to $MC7337" | tee -a $log
            tapeutil -f/dev/$drv ucode $microDir/$MC7337
         fi
      fi
      let i=i-1
   done

   # upgrade drive microcode
   lvl="0060"
   let i=4   #maximum of drives is 4 with posibility of 4mm being wrong place 
   while [ "$i" -ge 0  ]
   do
      drv=$(lsdev -Cc tape | grep "DLT" | grep "rmt$i" | awk '{ print $1 }' ) 
      if [ "$drv" != '' ]
      then
         need=$(tapeutil -f /dev/$drv vpd | grep "Revision Level" | grep $lvl )
         if [ "$need" = '' ]
         then
            echo "INFO: Upgrading drive $drv microcode to $lvl" | tee -a $log
            /tmp/genucode -s $drv
         fi
      fi
      let i=i-1
   done

   echo "mc_7337 " >> $rec_log
   echo "INFO: leaving mc_7337" | tee -a $log
}

# --------------------------------------------------------------------
# mc_3590:  add in mc for 3590 drive and  3494 library 
# --------------------------------------------------------------------
function mc_3590 {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"mc_3590 " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   # upgrade drive code
   echo "INFO: entering mc_3590" | tee -a $log
   list=$(lsdev -Cc tape | grep rmt | awk '{ print $1 }')      #max number of 3590 drives is 10
   for i in $list  
   do
      #  need to make sure not blythe ultra drive.  If blyth do differently
      #ultra=$(lscfg -vl $i | grep "Loadable Microcode Level" | cut -c 37-)
      #if [ "$ultra" = "AOB00E22" ]
      ultra=$(tapeutil -f/dev/$i vpd | grep "Revision Level" | awk '{ print $4 }' | cut -c 1-1 )
      if [ "$ultra" = "B" ]    #regular drives will start with 9  
      then
         echo "INFO: $i is a blythe ultra drive." | tee -a $log
         MC=$MC3590b
         lvl="B88C"
      elif [ "$ultra" = "C" [
      then 
         MC=$MC3590c
         lvl="B88C"
      else
         MC=$MC3590a
         lvl="A4C6"
      fi
      thr=$(lsdev -Cc tape | grep "$i ")
      if [ "$thr" != '' ]
      then
         need=$(tapeutil -f /dev/$i vpd | grep "Revision Level" | awk '{ print $4 }')
         if [ "$need" != $lvl ]
         then
            echo "INFO: /dev/$i is at microcode level $need." | tee -a $log
            echo "      will updated code to $MC" | tee -a $log
            tapeutil -f/dev/$i ucode $microDir/$MC
         else
            echo "INFO: /dev/$i already at level $need." | tee -a $log
         fi
      fi
   done

   echo "mc_3590 " >> $rec_log
   echo "INFO: leaving mc_3590" | tee -a $log
}


# --------------------------------------------------------------------
# postbackup_run:  run postbackup  
# --------------------------------------------------------------------
function postbackup_run
{

   echo "INFO: entering postbackup_run" | tee -a $log
   postbackup importit >> /dev/null 2>> /dev/null
   rc=$?
   if [ "$rc" != '0' ]
   then
      echo "ERROR:  'postbackup importit' did not complete successfully." | tee -a $log
      echo "    fix the error and restart this script." | tee -a $log
      exit 1
   fi  
 

   echo "INFO: leaving postbackup_run" | tee -a $log
}

# --------------------------------------------------------------------
# prebackup_run:  run prebackup  
# --------------------------------------------------------------------
function prebackup_run
{

   echo "INFO: entering prebackup_run" | tee -a $log

   prebackup exportit >> /dev/null 2>> /dev/null
   rc=$?
   if [ "$rc" != '0' ]
   then
      echo "ERROR:  prebackup did not complete successfully." | tee -a $log
      echo "    fix the error and restart this script." | tee -a $log
   fi  

   echo "INFO: leaving prebackup_run" | tee -a $log
}



# --------------------------------------------------------------------
# record_lsSSlevel: record lsSSlevel files to /adsmlog 
# --------------------------------------------------------------------
function record_lsSSlevel {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"record_lsSSlevel" $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi
  echo "INFO:  entering record_lsSSlevel" | tee -a $log
  if [ -f /var/adm/WebSh* ]
  then
     adsmlog_full
     if [ "$ADSMLOG_FULL" = "NO" ]
     then
        dir="$dirForFilesInAdsmlog"
     else
        echo "INFO: must record to diskette no room on adsmlog." | tee -a $log
        dir="/tmp/record"
     fi
     echo "INFO:  Copying lsSSlevel files to $dir" | tee -a $log
     cp /var/adm/WebSh* $dir/ >> /dev/null 2>>/dev/null
     rc=$?
     if [ "$rc" != '0' ]
     then
        echo "ERROR: could not copy  /var/adm/WebSh* to $dir" | tee -a $log
        echo "    Fix problem, then rerun this script."
        exit 1
     else
        echo "lsSSlevel files recorded" | tee -a $log
        echo "record_lsSSlevel_dir $dir" >> $rec_log
     fi
   else
     echo "WARNING: no lsSSlevel files are present." | tee -a $log
   fi 
   echo "record_lsSSlevel" >> $rec_log
   echo "INFO:  leaving record_lsSSlevel" | tee -a $log

}
  

# --------------------------------------------------------------------
# redefine_tape: Invoke our def routines to define tape to aix.
# --------------------------------------------------------------------
function redefine_tape {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"TAPEDEFINED" $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
   echo "INFO: entering redefine_tape" | tee -a $log

   # Verify server is down
   adsmdown

   is3575=$(lsdev -Cc tape | grep -c 3575 )
   is3590=$(lsdev -Cc tape | grep -c 3590 )
   is7337=$(lsdev -Cc tape | grep -c 7337 )

   let rc=0
   let rc1=0
   let rc2=0
   if [ "$is3575" -gt '0' ]
   then
      echo "INFO: this is a 3575 - code will be updated manually." | tee -a $log
   elif [ "$is3590" -gt '0' ]
   then
      echo "INFO: this is a 3494 - code will be updated manually." | tee -a $log
   elif [ "$is7337" -gt '0' ]
   then
      mc_7337
   else
      echo "INFO:  this is not a 7337, 3575 or 3494.  " | tee -a $log
   fi
   if [ "$rc" != '0' -o "$rc2" != '0' -o "$rc1" != '0' ]
   then
      echo
      echo "*******************************************************************"
      echo "ERROR: could not correctly define the hardware." | tee -a $log
      echo "       Resolve the problem before continuing." | tee -a $log
      exit 1
   fi
   echo "TAPEDEFINED" >> $rec_log
   echo "INFO: leaving redefine_tape" | tee -a $log

}


# --------------------------------------------------------------------
# register_lic
# --------------------------------------------------------------------
function register_lic {

   echo "INFO: entering register_lic." >> $log

   drm_thr=$(grep "$server DRM_LIC" $rec_log | awk '{ print $3 }' )
   echo "INFO: server $server licensed for drm? $drm_thr" | tee -a $log          
   if [ "$drm_thr" = 'Yes' ]
   then
      dsmadmc $admInf register lic file=$tivolidir/drm.lic
      echo "INFO: just registered drm license for server $server." | tee -a $log
   else
      echo "INFO: not registering drm lic for $server" | tee -a $log
   fi
   hsm_thr=$(grep "$server HSM_LIC" $rec_log | awk '{ print $3 }' )
   echo "INFO: server $server licensed for hsm? $hsm_thr" | tee -a $log          
   if [ "$hsm_thr" = 'Yes' ]
   then
      dsmadmc $admInf register lic file=$tivolidir/spaceman.lic
      echo "INFO: just registered hsm license for server $server." | tee -a $log
   else
      echo "INFO: not registering hsm lic for $server" | tee -a $log
   fi
   echo "INFO: about to register other licenses for server $server." | tee -a $log
   echo "INFO: about to register other licenses for server $server." | tee -a $log
   dsmadmc $admInf register lic file=$tivolidir/network.lic
   dsmadmc $admInf register lic file=$tivolidir/opensys.lic
   dsmadmc $admInf register lic file=$tivolidir/advdev.lic
   dsmadmc $admInf -outfile=/tmp/lic -displaymode=list q lic 
   numclient=$(cat /tmp/lic | grep "Licensed Client Nodes:" | awk '{ print $4 }' | tr -d ',' )
   echo "INFO: $numclient already registered" | tee -a $log 
   let numclient=numclient+0
   while [ $numclient -lt 5001 ]
   do
      dsmadmc $admInf register lic file=$tivolidir/50client.lic
      dsmadmc $admInf -outfile=/tmp/lic -displaymode=list q lic 
      numclient=$(cat /tmp/lic | grep "Licensed Client Nodes:" | awk '{ print $4 }' | tr -d ',' )
      echo "INFO: $numclient already registered" | tee -a $log 
   done
   echo "INFO: leaving register_lic." >> $log
}
# --------------------------------------------------------------------
# --------------------------------------------------------------------
# remove_adsmlog:  remove the redirection of server to adsmlog
# --------------------------------------------------------------------
function remove_adsmlog {
   echo "INFO: entering remove_adsmlog" | tee -a $log
   proc=$(ps -eaf | grep dsmserv | grep dsmulog | awk '{ print $2 }')
   if [ "$proc" != '' ]
   then
      echo "INFO: killing redirect to adsmlog process $proc" | tee -a $log
      kill -9 $proc
   else
      echo "INFO: no adsmlog redirect process active." | tee -a $log
   fi
   echo "INFO: deleting any /adsmlog/output.log enteries. " | tee -a $log
   rm /adsmlog/output.log* >> /dev/null 2>> /dev/null
   echo "INFO: leaving remove_adsmlog" | tee -a $log
}

# --------------------------------------------------------------------
# remove_extra_files
# --------------------------------------------------------------------
function remove_extra_files {
   echo "INFO: entering remove_extra_files" | tee -a $log

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"remove_extra_files " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
   for file in $extra_files
   do
     if [ -f $file ]
     then
        echo "INFO: removing $file"
        rm $file >> /dev/null 2>>/dev/null
     fi
   done

   for dir in $extra_dirs
   do
     if [ -d $dir ]
     then
        echo "INFO: removing $dir"
        rmdir $dir >> /dev/null 2>>/dev/null
     fi
   done
   
   echo "remove_extra_files " >> $rec_log
   echo "INFO: leaving remove_extra_files" | tee -a $log
}


# --------------------------------------------------------------------
# reset_admInf:  Set up admInf based on this being phase 2
# --------------------------------------------------------------------
function reset_admInf {

  echo "INFO: entering reset_admInf" | tee -a $log

  str=$(grep "id=" $rec_log)
  export admInf="$str"

  echo "INFO: leaving reset_admInf" >> $log
}


# --------------------------------------------------------------------
function run_record {

  if [ "$reRunScript" = "yes" ]
  then
    str=''
    str=$(grep ^"RUN_RECORD " $rec_log)
    if [ "$str" != '' ]
    then
       return 0
    fi
  fi

  echo "INFO: entering run_record" | tee -a $log
  echo "INFO: running the record script to record off all disk info." | tee -a $log

  record.ssa all

  # this records off disk info needed for record.build.script
  echo "Issueing command: record 1 $adminid $adminpw $ssm"
  record 1 $adminid $adminpw $ssm

  # this creates the record.build.script
  echo "Issueing command: record 2 $adminid $adminpw $ssm"
  record 2 $adminid $adminpw $ssm

  
  echo "RUN_RECORD " >> $rec_log
  echo "INFO: leaving run_record" | tee -a $log
}


function run_fix {
   echo "INFO: entering run_fix" | tee -a $log
   bosfile="/var/adm/ras/bosinst.data"
   if [[ ! -f $bosfile.orig ]]
   then
      cp $bosfile $bosfile.orig
   fi
   old="PROMPT = no"
   new="PROMPT = yes"
   sed "s/$old/$new/" < $bosfile.orig > $bosfile
   aixrc=$?
   if [ "$aixrc" != 0 ]
   then
      Msg="Could not update $bosfile."
      echo "ERROR: $aMsg" | tee -a $log
      return 1
   else
      cp $bosfile $bosfile.orig
   fi

   echo "INFO: leaving run_fix" | tee -a $log
}


# --------------------------------------------------------------------
# save_lic
# --------------------------------------------------------------------
function save_lic {

   echo "INFO: entering save_lic." >> $log
   for server in $server_list
   do
      which_server $server
      dsmadmc $admInf -outfile=/tmp/lic -displaymode=list q lic 
      drm_thr=$(cat /tmp/lic | grep " Is disaster recovery manager licensed" | awk '{ print $7 }' )
      thr=$(grep "$server DRM_LIC" $rec_log)
      if [ "$thr" = '' ]
      then
         echo "$server DRM_LIC $drm_thr" >> $rec_log         
      fi
      echo "INFO: server $server licensed for drm? $drm_thr" | tee -a $log          
      hsm_thr=$(cat /tmp/lic | grep "Is space management licensed" | awk '{ print $6 }' )
      thr=$(grep "$server HSM_LIC" $rec_log)
      if [ "$thr" = '' ]
      then
         echo "$server HSM_LIC $hsm_thr" >> $rec_log         
      fi
      echo "INFO: server $server licensed for hsm? $hsm_thr" | tee -a $log          
   done
   echo "INFO: leaving save_lic." >> $log
}
# --------------------------------------------------------------------
# set_servdir
# --------------------------------------------------------------------
function set_servdir {

   echo "INFO: entering set_servdir." >> $log

   echo "INFO: obtaining level of ADSM code loaded." | tee -a $log
   tivoli=$(lslpp -l | grep -i "tivoli.tsm.server.com")
   if [ "$tivoli" = '' ]
   then
      export servdir="/usr/lpp/adsmserv/bin"
   else
      export servdir="/usr/tivoli/tsm/server/bin"
   fi
   
   str=`lslpp -l all | grep tivoli.tsm.client.ba`
   if [ "$str" != '' ]
   then
      echo "INFO:  tivoli client installed  "
      export clientdir="/usr/tivoli/tsm/client/ba/bin"
   else
      export clientdir="/usr/lpp/adsm/bin"
   fi

   echo "INFO: leaving set_servdir." >> $log
}

# --------------------------------------------------------------------
# start_adsm
# --------------------------------------------------------------------
function start_adsm {
  which_serv=$1
  echo "INFO: entering start_adsm" | tee -a $log
  echo "INFO: issuing startadsm $which_serv" | tee -a $log
  startadsm $which_serv >> /dev/null 2>> /dev/null
  echo "INFO: Waiting for ADSM to start." | tee -a $log
  sleep 20

  let cont=0
  while [ "$cont" -eq '0' ]
  do
     dsmadmc $admInf quit >> /dev/null 2>> /dev/null
     rc=$?
     if [ "$rc" = "137" ]
     then
        echo
        echo "*****************************************************************"
        echo "WARNING: your password has expired." | tee -a $log
        echo "  In order for this script to work you must update your password." | tee -a $log
        echo "  You will need to call support to work around this problem."
        echo "  Press <Enter> to continue" | tee -a $log
        read
     elif [ "$rc" = "-451" ]
     then
        echo
        echo "*****************************************************************"
        echo "WARNING: your password may have expired." | tee -a $log
        echo "  In order for this script to work you may need to update your password." | tee -a $log
        echo "  You will need to call support to work around this problem."
        echo "  Press <Enter> to continue" | tee -a $log
        read
     elif [ "$rc" -ne '0' ]
     then
        echo "INFO: checking to make sure still coming up correctly." | tee -a $log
        echo "INFO: dsmadmc $admInf quit came back with rc=$rc" | tee -a $log 
        out=$(ps -eaf | grep -v grep | grep -v isitup | grep -c dsmserv)
        if [ "$out" -eq '0' ]
        then
           echo "INFO: ADSM server did not start correctly. Retrying." | tee -a $log
           curDir=`pwd`   #capture current directory
           cd $dsmserv 
           dsmserv > /dev/null 2>&1 &
           cd $curDir        
        fi

        echo "INFO: Sleeping another 30 seconds.  Waiting for ADSM server to come up." | tee -a $log
        echo "INFO: issuing 'dsmadmc $admInf quit'" | tee -a $log
        sleep 30
     else
        echo "INFO: ADSM server is up." | tee -a $log
        let cont=1
     fi
  done
  echo "INFO: leaving start_adsm" | tee -a $log
}


# --------------------------------------------------------------------
# start_idl: must start web admin interface for each server on machine
# --------------------------------------------------------------------
function start_idl {
  echo "INFO: entering start_idl" | tee -a $log

  for server in $server_list
  do
      export server=$server
      which_server $server
      admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
      if [ "$admInf" = '' ]
      then
         admin_id $server
      fi
      export admInf=$admInf
      echo "INFO: starting idl for server $server" | tee -a $log
      cd $adsmdir
      dsmserv runfile /usr/tivoli/tsm/server/webimages/dsmserv.idl
  done
     
  echo "INFO: leaving start_idl" | tee -a $log
}

# --------------------------------------------------------------------
# Tidy_Up:  Move pre ga code around.
# --------------------------------------------------------------------
function TidyUp
{
        # move microcode files back to /etc/microcode
        mv /tmp/disk45/ssadisk.ros.* /etc/microcode >/dev/null
        # make sure that level 46 and 47 are not left in /etc/microcode
        rm /etc/microcode/ssadisk.ros.*.46 >/dev/null
        rm /etc/microcode/ssadisk.ros.*.47 >/dev/null
        exit 1
}


# --------------------------------------------------------------------
# type_tape:  Determine what type tape is attached and how many libs.
# --------------------------------------------------------------------
function type_tape {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"is3575" $rec_log)
     if [ "$str" != '' ]
     then
        devn=$(cat $rec_log | grep ^devn | awk '{ print $2 }')
        libn=$(cat $rec_log | grep ^"libn " | awk '{ print $2 }')
        export devn=$devn
        export libn=$libn
        return 0
     fi
   fi
   echo "INFO: entering tape_type" | tee -a $log

   #--------------------------------------------------
   # record how many libraries attached
   #--------------------------------------------------
   numlib=$(dsmadmc $admInf q library | grep -c "/dev/")

   #--------------------------------------------------
   # is this for 3575, 3590 or 7337?
   #--------------------------------------------------
   numb=$(lsdev -Cc tape | grep -i -c Tape)
   if [ "$numb" -lt '1' ]
   then
     cfgmgr
   fi
   is3590=$(lsdev -Cc tape | grep -c 3590)
   is7337=$(lsdev -Cc tape | grep -c 7337)
   is3575=$(lsdev -Cc tape | grep -c 3570)
   if [ "$is3575" -gt '0' ]
   then
     echo "INFO: 3575 detected." | tee -a $log
     libn="3575lib"
     libn2="3575lib2"
     devn="3570"
   elif [ "$is7337" -gt '0' ]
   then
     echo "INFO: 7337 detected." | tee -a $log
     libn="7337lib"
     libn2="7337lib2"
     devn="DLT"
   elif [ "$is3590" -gt '0' ]
   then
     echo "INFO: 3590s detected." | tee -a $log
     libn="3494lib"
     libn2=" "
     devn="3590_CLASS"
   else
     clear
     echo "WARNING: did not detect a 3575, 7337 or 3590 when issuing lsdev -Cc tape." | tee -a $log
     echo "         This may be normal if this is an A00 or C00." | tee -a $log
     libn="JUNK"
     echo "Press <ENTER> to continue."
     read
   fi
   echo "NUMLIB $numlib" >> $rec_log
   export libn=$libn
   export libn2=$libn2
   export devn=$devn
   echo "devn $devn" >> $rec_log
   echo "is3590 $is3590" >> $rec_log
   echo "is7337 $is7337" >> $rec_log
   echo "is3575 $is3575" >> $rec_log
   echo "libn $libn" >> $rec_log
   echo "libn2 $libn2" >> $rec_log
   echo "INFO: leaving tape_type" | tee -a $log

}


# --------------------------------------------------------------------
# update_addauto  make sure addauto has new paths and such
# --------------------------------------------------------------------
function update_addauto {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"update_addauto " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   echo "INFO: entering update_addauto" | tee -a $log
 
   echo "INFO: issuing removeauto" | tee -a $log
   rmitab "autosrvr" >> /dev/null 2>> /dev/null

   echo "INFO: issuing removetrace" | tee -a $log
   rmitab "autotrace" >> /dev/null 2>> /dev/null

   echo "INFO: issuing addauto " | tee -a $log
   mkitab autosrvr:2:once:"/usr/nsm.3466/script/startadsm all"

   echo "update_addauto " >> $log 
   echo "INFO: leaving update_addauto" | tee -a $log

}

# --------------------------------------------------------------------
# update_dsmservopt: updates the option files devconfig and volhist 
# --------------------------------------------------------------------
function update_dsmservopt {

   echo "INFO: entering update_dsmservopt" | tee -a $log

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"update_dsmservopt " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi

   cp $tivolidir/dsmserv.opt $tivolidir/dsmserv.orig
   old="\/usr\/lpp\/adsmserv\/bin\/volhist.file"
   new="\/usr\/tivoli\/tsm\/server\/bin\/volhist.file"
   echo "INFO: issuing sed s/$old/$new < $tivolidir/dsmserv.orig > $tivolidir/dsmserv.opt" | tee -a $log
   sed "s/$old/$new/" < $tivolidir/dsmserv.orig > $tivolidir/dsmserv.opt

   cp $tivolidir/dsmserv.opt $tivolidir/dsmserv.orig
   old="\/usr\/lpp\/adsmserv\/bin\/devconfg.file"
   new="\/usr\/tivoli\/tsm\/server\/bin\/devconfg.file"
   echo "INFO: issuing sed s/$old/$new < $tivolidir/dsmserv.orig > $tivolidir/dsmserv.opt" | tee -a $log
   sed "s/$old/$new/" < $tivolidir/dsmserv.orig > $tivolidir/dsmserv.opt


   dsmservthr=$(grep "adsmserv" $tivolidir/dsmserv.opt)
   if [ "$dsmservthr" != '' ]
   then
      clear
      echo "*******************************************************************"
      echo "WARNING: the following are still showing up in $tivolidir/dsmserv.opt" | tee -a $log
      echo $dsmservthr
      echo
      echo "   We would have expected all references to the old ADSM directory" | tee -a $log
      echo "   to have been updated to /usr/tivoli/tsm/server/bin" | tee -a $log
      echo "   You should verify this with the customer and have them make needed changes." | tee -a $log
      echo  
      PS3="Select Option: "
      select option in \
      '    Continue, I understand what needs to be done ' \
      '    EXIT.'
      do
          case $REPLY in
          1 ) echo
              break;;
          2 ) exit 1;;
          * ) echo "INVALID selection";;
          esac
      done
   fi

   echo "update_dsmservopt " >> $rec_log 
   echo "INFO: leaving update_dsmservopt" | tee -a $log
}

# --------------------------------------------------------------------
# update_mc mirocode for h50 AIX system and service processor 
# --------------------------------------------------------------------
function update_mc {

   echo "INFO: entering update_mc" | tee -a $log
   if [ "$processorType" = "H50" ]
   then
     systemMc=$systemMCh50
     systemLVL=$systemLVLh50
     echo "INFO: This program will update the H50 system and service processor microcode." | tee -a $log
     echo "INFO: Copying microcode to $dirForMc" | tee -a $log
     if [ ! -d $dirForMc ]
     then
          echo "INFO: creating $dirForMc" | tee -a $log
          mkdir $dirForMc
     fi
     cp $microDir/$systemMc $dirForMc/ >> /dev/null 2>> /dev/null
     rc=$?
     if [ "$rc" != 0 ] 
     then
          echo "ERROR:  Could not copy firmware to $dirForMc."
          echo "        Resolve problem and restart script."
          exit 1
     fi
     cp $microDir/$spMc $dirForMc/ >> /dev/null 2>> /dev/null
     rc=$?
     if [ "$rc" != 0 ] 
     then
          echo "ERROR:  Could not copy firmware to $dirForMc."
          echo "        Resolve problem and restart script."
          exit 1
     fi
  
     let cont=0
     while [ "$cont" = '0' ]
     do
        clear
        echo
        echo "********************************************************************"
        echo "WARNING: if there is not enough space in /var this process will fail."
        echo
        echo "Do you want to update system or service processor code?"
        echo " 1) System microcode. THIS MUST be done first."
        echo " 2) Service processor mc. THIS SHOULD ONLY be done after system mc."
        read cont
        case $cont in
        1) echo | tee -a $log
           echo "To update the system firmware do the following steps." | tee -a $log
           echo "1) Open another AIX window." | tee -a $log
           echo "2)  Then issue the commands:                                " | tee -a $log
           echo "       cd /usr/lpp/diagnostics/bin                        "
           echo "       ./update_flash -f $dirForMc/$systemMc " | tee -a $log
           read  
           echo ;;
  
        2) echo  | tee -a $log
           thr=$(lscfg -vp | grep -p 'System Firmware' | grep ROM | awk '{ print $3 }' | tr '.' ' ' | awk '{ print $2 }') 
           if [ "$thr" = '' ]      # 2 different formats for output of command
           then
              thr=$(lscfg -vp | grep -p 'System Firmware' | grep ROM | awk '{ print $2 }' | tr '.' ' ' | awk '{ print $3 }') 
           fi
           if [ "$thr" = "$systemLVL" ]
           then
              echo "To update the service processor firmware do the following steps." | tee -a $log
              echo "1) Open another AIX window." | tee -a $log
              echo "2  Then issue the commands:                                " | tee -a $log
              echo "       cd /usr/lpp/diagnostics/bin" | tee -a $log
              echo "       ./update_flash -f $dirForMc/$spMc " | tee -a $log
              read ;
           else
              echo "ERROR: the system firmware is currently at $thr" | tee -a $log
              echo "   It must be at $systemLVL in order to update the sp mc." | tee -a $log 
              exit 1
           fi
           echo ;;
         *) let cont=0;;
         esac
     done 
     cd $currentDir
   fi

   if [ "$processorType" = "H10" ]
   then
      systemMc=$systemMCh10
      systemLVL=$systemLVLh10
      clear
      echo "******************************************************************"
      echo "You need to manually upgrade the system firmware." | tee -a $log
      echo "microcode for the H10."
      echo
   fi
 
   if [ "$processorType" = "R20" ]
   then
      echo "******************************************************************"
       echo "INFO: there is no system microcode updates needed on the R20" | tee -a $log
   fi
   echo "INFO: leaving update_mc" | tee -a $log
}

# --------------------------------------------------------------------
# update_profile:  update the 3466ENV and profile
# --------------------------------------------------------------------
function update_profile {

   echo "INFO: entering update_profile" | tee -a $log
   if [ -f /3466Env ]
   then
      if [ -f /usr/nsm.3466/script/3466Env ]
      then
         echo "INFO: saving off /3466Env, as it should be in /usr/nsm.3466/script" | tee -a $log
         mv /3466Env /3466Env.old
      else
         echo
         echo "******************************************************************"
         echo "ERROR: there is no 3466Env in /usr/nsm.3466/script." | tee -a $log
         echo "   Make sure the latest nsm.3466 package is installed. " | tee -a $log
         exit 1
      fi
   fi
  
   thr=$(cat /.profile | grep "/usr/nsm.3466/script/3466Env" )
   if [ "$thr" = '' ]
   then 
      echo "INFO: putting in new .profile to point to new /usr/nsm.3466/script/3466Env." | tee -a $log
      mv /.profile /.profile.before.ec
      cp $microDir/.profile  /.profile
      rc=$?
      if [ "$rc" != '0' ]
      then
          echo "**********************************************************"
          echo "ERROR: could not copy $microDir/.profile /.profile" | tee -a $log
          echo "Resolve, then restart." | tee -a $log
          exit 1
      fi
   fi
    

   echo "INFO: leaving update_profile" | tee -a $log

}

# --------------------------------------------------------------------
# update_ssa mirocode ssa code 
#     this has to be done before postbackup is run
# --------------------------------------------------------------------
function update_ssa {

   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"UPDATE_SSA " $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi


   echo "INFO: entering update_ssa" | tee -a $log
   vgon=$(lsvg | grep adsmvg)
   if [ "$vgon" != '' ]
   then
      echo "INFO: having to run prebackup to export $vgon" | tee -a $log
      prebackup exportit >> /dev/null 2>> /dev/null
   fi

   # first untar stuff from cd
   mount /cd >> /dev/null 2>>/dev/null
   tar -xvf $microDir/$SSAdir/$ssaTarFile
   rc=$?
   if [ "$rc" != '0' ]
   then
      echo "ERROR: could not untar ssa code. rc=$rc." | tee -a $log
      echo "   Press <ENTER> to continue."
      read
      return
   fi

   # now do load of code
   cmd="ssadload -u"
   echo "INFO: issuing $cmd" | tee -a $log
   $cm
   echo "UPDATE_SSA " >>$rec_log
   postbackup_run

    
   echo "INFO: leaving update_ssa" | tee -a $log
}


# --------------------------------------------------------------------
# update_model this makes sure the 3466.model file is up to date
# --------------------------------------------------------------------
function update_model {
   if [ "$reRunScript" = "yes" ]
   then
     str=''
     str=$(grep ^"update_model" $rec_log)
     if [ "$str" != '' ]
     then
        return 0
     fi
   fi
   echo "INFO: entering update_model" | tee -a $log
   # first make sure 4.3.2 came back
   lvl=$(oslevel)
   if [ "$lvl" != "$OSLEVEL" ]
   then
      echo "ERROR: $lvl AIX is loaded it should be $OSLEVEL." | tee -a $log
      echo "  Something failed in loading the mksysb." | tee -a $log
      exit 1
   else
      echo "INFO: $OSLEVEL AIX is loaded." | tee -a $log
   fi
   theModel=$(cat $rec_log |grep "model is" | awk '{ print $3 }')
   dt=$(date)  
   if [ "$theModel" = "MODELALREADYTHERE" ]
   then
      echo "INFO: 3466.model already correct." | tee -a $log 
   else
      str=$(lsdev -C | grep pdisk0 | grep "4GB SSA" )
      if [ "$str" != '' ]
      then
        diskType="4GB"
      else
        diskType="9GB"
      fi        
      rm $model_file >> /dev/null 2>> /dev/null
      echo "$theModel, $diskType disks, updated to $ECLEVEL on $dt" > /usr/nsm.3466/sysInfo/3466.model
   fi                                                             
     
   echo "update_model" >> $rec_log
   echo "INFO: leaving update_model" | tee -a $log
} 


# --------------------------------------------------------------------
# verify_adsm_lvl: Verify that the adsm levels are the same
# --------------------------------------------------------------------
function verify_adsm_lvl {

  echo "INFO: entering verify_adsm_lvl" | tee -a $log
      
  adsm_lvl
  vrmf=$(cat $rec_log | grep ^"ADSM Server" | awk ' { print $3 }' )
  echo ${vrmf} | sed -e "s/\./ /g" | read VER REL MOD FIX
  vrmfold=$(cat $rec_log | grep ^"ADSM Server" | awk ' { print $3 }' )
  echo ${vrmfold} | sed -e "s/\./ /g" | read VEROLD RELOLD MODOLD FIXOLD

  bad_lvl="no"
  if [[ "$REL" < "$RELOLD" ]]
  then
     bad_lvl="yes"
  elif [[ "$REL" = "$RELOLD" ]] 
  then
     if [[ "$MOD" < "$MODOLD" ]]
     then
        bad_lvl="yes"
     elif [[ "$MOD" = "$MODOLD" ]] 
     then
         if [[ "$FIX" < "$FIXOLD" ]] 
         then
            bad_lvl="yes"
         fi
     fi
  fi
  if [ "$bad_lvl" = "yes" ]
  then
     clear
     echo
     echo "****************************************************************"
     echo "ERROR: The new level of ADSM does not match the previous level:" | tee -a $log
     echo "       NEW LEVEL: $vrmf" | tee -a $log
     echo "       OLD LEVEL: $vrmfold" | tee -a $log
     echo "       You should update the level either now or later."
     let cont=0
     while [ "$cont" -eq '0' ]
     do
       echo "**********************************************************************"
       echo "1 ) Continue and update level later." | tee -a $log
       echo "2 ) Stop and update now." | tee -a $log

       read cont
       case $cont in
       1 ) echo "INFO: You will need to update the adsm server level to $vrmfold" | tee -a $todo
           ;;
       2 ) exit 1;;
       * ) let cont=0;;
       esac
     done
  fi
  echo "INFO: leaving verify_adsm_lvl" | tee -a $log
}

#----------------------------------------------------------------
#  if this was a v32 server, must convert to v37
#----------------------------------------------------------------
function verify_not_v32 {
   echo "INFO: entering verify_not_v32" | tee -a $log
   curDir=`pwd`
   old_lvl=$(cat $rec_log | grep V31OLDADSMLEVEL )
   if [ "$old_lvl" != '' ]
   then
      for server in $server_list
      do
         which_server $server
         admInf=$(cat $rec_log | grep "ADMINF$server " | sed -e "s/ADMINF$server/ /" )
         if [ "$admInf" = '' ]
         then
            admin_id $server
         fi
         export admInf=$admInf
         echo "INFO: having to issue upgradedb command for v2 or v3.2.1 server $server" | tee -a $log
         echo "      from dirctory $adsmdir." | tee -a $log
         cd $adsmdir
         nohup dsmserv upgradedb > /dev/null 2>&1 &
         rc=$?
         if [ "$rc" != '0' ]
         then
            if [ "$server" != "primary"
            then
               echo "WARNING: unable to upgrade the db on server $server." | tee -a $log
               PS3="Select Option: "
               select option in \
                '    Continue, this kin server should not have been updated. ' \
                '    EXIT, this kin server should have been updated. CE must investigate error.'
               do
                  case $REPLY in
                  1 ) echo
                      break;;
                  2 ) exit 1;;
                  * ) echo "INVALID selection";;
                  esac
               done
            else
               echo "ERROR: could not update primary server's database." | tee -a $log
               echo "   Issued:  cd $adsmdir" | tee -a $log
               echo "            nohup dsmserv upgradedb > /dev/null 2>&1 &" | tee -a $log
               exit 1
            fi
         else
            echo "INFO: just issued upgrade db for server $server." | tee -a $log
            let cont=0
            while [ "$cont" = 0 ]
            do
              dsmadmc $admInf quit >> /dev/null 2>> /dev/null
              rc=$?
              if [ "$rc" != '0' ]
              then
                 echo "INFO: Sleeping another 30 seconds.  Waiting for ADSM server to come up. " | tee -a $log
                 echo "INFO: issuing 'dsmadmc $admInf quit'" | tee -a $log
                 sleep 30
              else
                 echo "INFO: ADSM server $server is up." | tee -a $log
                 let cont=1
              fi
            done
            cd $curDir
            halt_adsm
         fi
      done
   else
      echo "INFO: already had tivoli storage manager installed." | tee -a $log
      echo "      No need to upgrade server $server." | tee -a $log
   fi

   # nothing needs to be done on web this time as we do start_idl what needs to be upgraded on web anything?
   #old_lvl=$(cat $rec_log | grep V2OLD )
   #if [ "$old_lvl" != '' ]
   #then
      ##--------------------------------------------------------
      ##  run updateweb
      ##--------------------------------------------------------
      #echo
      #echo "INFO: Running updateweb." | tee -a $log
      #/usr/lpp/StorageSmart/install/ws_sethn -a >> /dev/null
      #stopsrc -s httpd
      #sleep 15
      #startsrc -s httpd
#
      ##--------------------------------------------------------
      ## update web stuff
      ##--------------------------------------------------------
      #cd $dsmserv 
      #cmd="nohup dsmserv runfile dsmserv.idl &"
      #echo "Issuing: $cmd" | tee -a $log
      #$cmd
      #rc=$?
      #sleep 20
      #cd $curDir
   #fi
   echo "INFO: leaving verify_not_v32" | tee -a $log
}


#----------------------------------------------------------------
#  make sure ss-m is present in dsm.sys file 
#----------------------------------------------------------------
function verify_dsmsys {
   echo "INFO: entering verify_dsmsys" | tee -a $log
   ssm=$(grep "ss-m" $clientdir/dsm.sys)
   if [ "$ssm" = '' ]
   then
      echo "Adding serverid ss-m to $clientdir/dsm.sys."
      echo "SErvername  ss-m" >> $clientdir/dsm.sys
      echo "COMMmethod         sharedmem" >> $clientdir/dsm.sys
      echo "TXNBYTE            2000" >> $clientdir/dsm.sys
      echo "compression        off" >> $clientdir/dsm.sys
   fi


   echo "INFO: leaving verify_dsmsys" | tee -a $log
}
#----------------------------------------------------------------
#  verify updates of critical files applied
#----------------------------------------------------------------
function verify_update_applied {
   echo "INFO: entering verify_update_applied" | tee -a $log
   for file in $criticalFixes 
   do
      fileset=$(echo $file | tr ':' ' ' | awk '{ print $1 }')
      #level=$(echo $file | tr ':' ' ' | awk '{ print $2 }')
      level="3.7"
      firstOccur=$(lslpp -l | grep $fileset )     
      levelapplied=$(echo $firstOccur | grep $fileset | awk '{ print $2 }')     
      if [ "$levelapplied" = '' ]
      then
         echo "****************************************************************"
         echo "ERROR: $file does not appear to be installed."
         echo 
      fi
      thr=$(echo $levelapplied | grep $level )     
      if [ "$thr" = '' ]
      then
         echo
         echo "**************************************************************"
         echo "ERROR: $fileset is not showing up at expected level $ADSMLEVEL" | tee -a $log
         echo "   it is showing up installed at level $levelapplied" | tee -a $log
         PS3="Select Option: "
         select option in \
          '    Continue, the current level is better than what was shipped with this EC ' \
          '    EXIT, and let us figure out why this was not applied.'
         do
            case $REPLY in
            1 ) echo
                break;;
            2 ) exit 1;;
            * ) echo "INVALID selection";;
            esac
         done
      else
         echo "INFO: $fileset applied at level $level" | tee -a $log
         echo "INFO: creating link for MESs between old and new directories." | tee -a $log
         mv /usr/lpp/adsmserv/bin /usr/lpp/adsmserv/bin.old
         ln -s /usr/tivoli/tsm/server/bin /usr/lpp/adsmserv/bin/
      fi
   done

   echo "INFO: leaving verify_update_applied" | tee -a $log
}

# --------------------------------------------------------------------
# what_model:  which model is this
# --------------------------------------------------------------------
function what_model {
#   lib=$1
   drv=$1
#   str=$(lsdev -Cc tape | grep -i changer  | grep -i $lib | awk '{ print $3 }' | tr ',' ' ' | awk '{ print $2 }' )
   str=$(tapeutil -f /dev/$drv vpd | grep "Product Ident" | awk '{ print $4 }' )
   if [ "$str" = "03570B1A" ]
   then
      # model 1 will have address like: 20-60-00-1,0
      #  model 1 is a B model drive
      mod="mod1"
   elif [ "$str" = "03570C1A" ]
   then
      # model 2 will have address like: 20-60-00-0,1
      #  model 2 is a C model drive
      mod='mod2'
   else
     let thr=0
     while [ "$thr" = '0' ]
     do
       echo "*************************************************"
       echo "Is $drv a model one or model two 3575?           "
       echo "1 ) model 1"
       echo "2 ) model 2"
       read thr
       case $thr in
       1) mod="mod1";;
       2) mod="mod2";;
       *) thr=0;;
       esac
     done
   fi
   export mod=$mod

}
#-----------------------------------------------------------------------
#      get the correct admInf for the current resource
#      needed to accomodate multiple servers
#-----------------------------------------------------------------------
function which_admInf {

   echo "INFO: entering which_admInf."
   resource=$1
   nameConv=$(echo $resource | cut -b3)
   which_server $nameConv
   admInf=$(cat $rec_log | grep ADMINF$server | sed -e "s/ADMINF$server/ /" )
   if [ "$admInf" = '' ]
   then
     admin_id $server
   fi
   export admInf=$admInf
   echo "INFO: talking with server $server" | tee -a $log
   echo "INFO: leaving which_admInf."
}

#-----------------------------------------------------------------------
#      determine which server the resource belongs to.  This is
#      needed to accomodate multiple servers
#-----------------------------------------------------------------------
function which_server {

   echo "INFO: entering which_server with $1." | tee -a $log
   nameConv=$1
   case $nameConv in
   a | primary )  server="primary"
                  adsmdir=$servdir
                  port="1500"
                  httpp="1580"
                  ssm="ss-m"
                  nameConv=a
                  shmport="1510";;
   adsmkin01 | e) server=adsmkin01
                  adsmdir="/adsmkin01"
                  ssm="ss-m01"
                  httpp="1581"
                  port="1501"
                  ssm="ss-m01"
                  nameConv=e
                  shmport="1511";;
   adsmkin02 | i )server=adsmkin02
                  adsmdir="/adsmkin02"
                  port="1502"
                  httpp="1582"
                  ssm="ss-m02"
                  nameConv=i
                  shmport="1512";;
  adsmkin03 | o ) server=adsmkin03
                  adsmdir="/adsmkin03"
                  port="1503"
                  httpp="1583"
                  ssm="ss-m03"
                  nameConv=o
                  shmport="1513";;
  adsmkin04 | u ) server=adsmkin04
                  adsmdir="/adsmkin04"

                  port="1504"
                  httpp="1584"
                  ssm="ss-m04"
                  nameConv=u
                  shmport="1514";;
  adsmkin05 | A ) server=adsmkin05
                  adsmdir="/adsmkin05"
                  port="1505"
                  httpp="1585"
                  ssm="ss-m05"
                  nameConv=A
                  shmport="1515";;
  adsmkin06 | E ) server=adsmkin06
                  adsmdir="/adsmkin06"
                  port="1506"
                  httpp="1586"
                  ssm="ss-m06"
                  nameConv=E
                  shmport="1516";;
  adsmkin07 | I ) server=adsmkin07
                  adsmdir="/adsmkin07"
                  port="1507"
                  httpp="1587"
                  ssm="ss-m07"
                  nameConv=I
                  shmport="1517";;
  adsmkin08 | 0 ) server=adsmkin08
                  adsmdir="/adsmkin08"
                  port="1508"
                  httpp="1588"
                  ssm="ss-m08"
                  nameConv=O
                  shmport="1518";;
  adsmkin09 | U ) server=adsmkin09
                  adsmdir="/adsmkin09"
                  port="1509"
                  httpp="1589"
                  ssm="ss-m09"
                  nameConv=U
                  shmport="1519";;
       * )        echo "ERROR: $server not currently supported." | tee -a $log
                  exit 1;;
   esac
   export server=$server
   export adsmdir=$adsmdir
   export port=$port
   export httpp=$httpp
   export ssm=$ssm
   export shmport=$shmport
   export nameConv=$nameConv

   echo "INFO: leaving which_server with $server." | tee -a $log
}


# --------------------------------------------------------------------
# worry about this server? or it is just there for db backup? 
# --------------------------------------------------------------------
function worry_about_server {

  echo "INFO: entering worry_about_server $server" | tee -a $log
  server=$1
  if [ "$reRunScript" = "yes" ]
  then
    concern=''
    concern=$(grep "WORRY_ABOUT $server " $rec_log | awk '{ print $1 }') 
    if [ "$concern" = '' ]
    then
       if [ "$server" != "primary" ]    # always do primary server
       then
          clear
          echo "******************************************************************"
          echo "Is there data on server $server that must be converted?"
          echo "Or is $server just a disastor recovery server that has no active data on it?"
          echo "Or was $server never formated, and should be ignored completley?"
          select choice in \
          '   Yes, $server must be protected and converted. ' \
          '   Yes, $server is just a disastor recovery server with no active data on it.' \
          '   Yes, $server was never formated and should be ignored completley. '
          do
             case $job in
             1 ) echo "WORRY_ABOUT $server " >>$rec_log 
                 concern="WORRY_ABOUT" 
                 break;;
             2 ) echo "SORTA_WORRY_ABOUT $server " >> $rec_log
                 concern="SORTA_WORRY_ABOUT" 
                 break ;;
             3 ) echo "DONT_WORRY_ABOUT $server " >> $rec_log
                 concern="DONT_WORRY_ABOUT"
                 break ;;
             * ) echo "INVALID SELECTION";;
             esac 
          done
       else
          echo "WORRY_ABOUT primary " >> $rec_log
          concern="WORRY_ABOUT"
       fi
    fi
  fi
  export concern=$concern 
  echo "INFO: entering worry_about_server $server" | tee -a $log
}

# ----------------------------------------------------------------
#  MAIN
# ----------------------------------------------------------------

clear
PS3="Select Option: "
echo "**************************************************************************"
echo " Choose the option to perform:" | tee -a $log
echo "**************************************************************************"
select job in \
'   Phase1:  Install system and service processor mc. ' \
'   Phase2:  Apply latest AIX, ADSM and microcode levels.' \
'   Abort this script. '
do
   case $REPLY in
   1 ) export phase="phase1"
       break;;
   2 ) export phase="phase2"
       break;;
   3 ) exit 0;;
   * ) echo "INVALID SELECTION";;
   esac 
done

# -----------------------------------------------------
# Need to create directory right away so console output can be saved
# there.
# -----------------------------------------------------
if [ ! -d $dirForFiles ]
then
   mkdir $dirForFiles
fi

# -----------------------------------------------------
# Ensure current path starts with required directories
# -----------------------------------------------------
currentPath=`env PATH`
firstCurrentPathEntry=`echo $currentPath | cut -f1 -d':'`
secondCurrentPathEntry=`echo $currentPath | cut -f2 -d':'`

# -----------------------------------------------------
# Add directories to the beginning of the path as required
# -----------------------------------------------------
temp3466bin='/tmp/3466/bin'
if [ "$firstCurrentPathEntry" = "$temp3466bin" ]
then # OK, this is what we need
  if [ "$secondCurrentPathEntry" = '.' ] || [ "$secondCurrentPathEntry" = './' ]
  then
    echo "INFO: PATH already contains $temp3466bin."
  else
     restOfPath=`echo $currentPath | cut -f2- -d':'`
     echo "INFO: adding $temp3466bin to PATH"
     export PATH="$temp3466bin:./:$restOfPath"
  fi
else
  if [ "$firstCurrentPathEntry" = '.' ] || [ "$firstCurrentPathEntry" = './' ]
  then # OK, this is what we need
    newPath="$temp3466bin:$currentPath"
  else
    newPath="$temp3466bin:./:$currentPath"
  fi
  echo "INFO: adding $temp3466bin to PATH"
  export PATH="$newPath"
fi

set_servdir
verify_dsmsys

if [ "$phase" = "phase1" ]
then
   #-----------------------------------------------------------------
   # install firmware for H50 service processor and system
   #-----------------------------------------------------------------
   log_info
   end_adsm all
   get_type_machine
   update_mc
   end_phase_1
fi 2>&1 | tee -a $consolePhase1

if [ "$phase" = "phase2" ]
then
   #-----------------------------------------------------------------
   # Determine if phase 2 is allowed, phase 2 is allowed only for H50
   #-----------------------------------------------------------------
   get_type_machine 
   if [ "$processorType" = 'H50' ]
   then
     echo 
     echo "Executing phase 2 for Cxx system (RS/6000 $processorType)"
   else
     if [ "$allowPhase2" = 'yes' ]
     then
       if [ "$processorType" = 'H10' ]
       then
         echo
         echo "Executing phase 2 for NSM Axx system (RS/6000 $processorType)"
         echo "This is allowed since environment variable allowPhase2='$allowPhase2'"
         echo
       elif [ "$processorType" = 'R20' ]
       then
         echo
         echo "Executing phase 2 for NSM Bxx system (RS/6000 $processorType)"
         echo "This is allowed since environment variable allowPhase2='$allowPhase2'"
         echo
       else
         echo 
         echo "ERROR: processor type is: $processorType. Valid processor types are: H10, R20 or H50"
         echo "exiting ......"
         echo
         exit 1
       fi
     else
       if [ "$processorType" = 'H10' ]
       then
         echo
         echo "Phase 2 for NSM Axx system (RS/6000 $processorType) is not supported"
         echo "exiting .........."
         echo
         exit 1
       elif [ "$processorType" = 'R20' ]
       then
         echo
         echo "Phase 2 for NSM Bxx system (RS/6000 $processorType) is not supported"
         echo "exiting .........."
         echo
         exit 1
       else
         echo 
         echo "ERROR: processor type is: $processorType. Valid processor types are: H10, R20 or H50"
         echo "exiting ......"
         echo
         exit 1
       fi
     fi
   fi

   #-----------------------------------------------------------------
   # install fixes to machine 
   #-----------------------------------------------------------------
   log_info
   echo "prepare.ec.phase2 started: `date`"
   check_ec
   str=$(grep ^PHASE2COMPLETE $rec_log)
   if [ "$str" = '' ]
   then
      full_fs
      det_model
      # pjj check_devconfg  # no longer need
      export server_list=$(cat $rec_log | grep ^server_list | sed -e "s/server_list/ /")
      thr=$(echo $server_list | grep primary)
      if [ "$thr" = '' ]
      then
         server_list=$(ls / | grep adsmkin | awk '{ print $1 }')
         server_list="primary $server_list"
         for server in $server_list
         do
            worry_about_server $server
            if [ "$concern" = "DONT_WORRY_ABOUT" ]
            then
               echo "INFO: not including $server in server_list." | tee -a $log
            else
               server_l="$server $server_l"
            fi
         done
         export server_list=$server_l
         thr=$(cat $rec_log | grep ^server_list)
         if [ "$thr" = '' ]
         then
            echo "server_list $server_list" >> $rec_log
         fi
      fi
      admin_info
      startadsm all
      db_backup_thr
      save_lic
      halt_adsm all
      adsm_lvl
      apply_adsm_updates
      verify_update_applied
      cleanup_adsm
      apply_updates
      set_servdir
      insertSecondCd
      hsm_there
      call_fixSSA
      update_ssa
      check_ssa
      full_fs
      update_addauto
      verify_not_v32
      theModel=$(whatmodel | awk '{ print $1 }' | tr -d ',' )
      copy_in_files
      redefine_tape
      update_dsmservopt
      start_idl
      update_profile
      startadsm all
      ec_stamp_unit
      run_record
      run_fix
      adsm_able
      for server in $server_list
      do
         which_server $server
         register_lic
      done
#     install_adobe # removed from MES
      end_it
 
      echo "PHASE2COMPLETE" >> $rec_log
   fi
fi 2>&1 | tee -a $consolePhase2
